@@ -85,6 +85,7 @@ void menu_save_as_cmd (void)		{ menu_cmd (CK_Save_As); }
 void menu_insert_file_cmd (void)	{ menu_cmd (CK_Insert_File); }
 void menu_quit_cmd (void)		{ menu_cmd (CK_Exit); }
 void menu_mark_cmd (void)		{ menu_cmd (CK_Mark); }
+void menu_markcol_cmd (void)		{ menu_cmd (CK_Column_Mark); }
 void menu_ins_cmd (void)		{ menu_cmd (CK_Toggle_Insert); }
 void menu_copy_cmd (void)		{ menu_cmd (CK_Copy); }
 void menu_move_cmd (void)		{ menu_cmd (CK_Move); }
@@ -147,6 +148,7 @@ static menu_entry FileMenuEmacs[] =
 static menu_entry EditMenu[] =
 {
     {' ', N_("&Toggle Mark       F3"), 'T', menu_mark_cmd},
+    {' ', N_("&Mark Columns    S-F3"), 'T', menu_markcol_cmd},
     {' ', "", ' ', 0},
     {' ', N_("toggle &Ins/overw Ins"), 'I', menu_ins_cmd},
     {' ', "", ' ', 0},
@@ -163,6 +165,7 @@ static menu_entry EditMenu[] =
 static menu_entry EditMenuEmacs[] =
 {
     {' ', N_("&Toggle Mark       F3"), 'T', menu_mark_cmd},
+    {' ', N_("&Mark Columns    S-F3"), 'T', menu_markcol_cmd},
     {' ', "", ' ', 0},
     {' ', N_("toggle &Ins/overw Ins"), 'I', menu_ins_cmd},
     {' ', "", ' ', 0},
@@ -400,9 +403,10 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 
     CGetHintPos (&x, &d);
 
-    CDrawMenuButton (catstrs (ident, ".editmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 14,
+    CDrawMenuButton (catstrs (ident, ".editmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 15,
 		     _(" Edit "),
 		     _("Toggle mark\tF3"), '~', menu_cmd, CK_Mark,
+		     _("Toggle mark columns\tC-b"), '~', menu_cmd, CK_Column_Mark,
 		     "", ' ', 0, 0,
 		     _("Toggle insert/overwrite\tIns"), '~', menu_cmd, CK_Toggle_Insert,
 		     "", ' ', 0, 0,
@@ -434,7 +438,7 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 
     CGetHintPos (&x, &d);
 
-    CDrawMenuButton (catstrs (ident, ".commandmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 9,
+    CDrawMenuButton (catstrs (ident, ".commandmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 11,
 		     _(" Command "),
 		     _("Goto line...\tM-l"), '~', menu_cmd, CK_Goto,
 		     "", ' ', 0, 0,
@@ -444,7 +448,9 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 		     _("Delete macro...\t"), '~', menu_cmd, CK_Delete_Macro,
 		     "", ' ', 0, 0,
 		     _("Insert date/time\tC-d"), '~', menu_cmd, CK_Date,
-		     _("Format paragraph\tM-p"), '~', menu_cmd, CK_Paragraph_Format
+		     _("Format paragraph\tM-p"), '~', menu_cmd, CK_Paragraph_Format,
+		     "", ' ', 0, 0,
+		     _("Refresh display\tC-l"), '~', menu_cmd, CK_Refresh
 	);
 /* Tool hint */
     CSetToolHint (catstrs (ident, ".commandmenu", 0), _("Macros and internal commands"));
