@@ -25,8 +25,10 @@
 #ifndef MIDNIGHT
 #include <X11/Xmd.h>		/* CARD32 */
 #include <X11/Xatom.h>
+#ifndef GTK
 #include "app_glob.c"
 #include "coollocal.h"
+#endif
 #include "editcmddef.h"
 #include "mousemark.h"
 #endif
@@ -49,113 +51,11 @@ void edit_destroy_callback (CWidget * w)
 	CError ("Trying to destroy non-existing editor widget.\n");
 }
 
+#ifdef GTK
+
+#else
 void link_hscrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton);
-
-extern int option_editor_bg_normal;
-void edit_tri_cursor (Window win);
-/* starting_directory is for the filebrowser */
-CWidget *CDrawEditor (const char *identifier, Window parent, int x, int y,
-	   int width, int height, const char *text, const char *filename,
-		const char *starting_directory, unsigned int options, unsigned long text_size)
-{
-    static made_directory = 0;
-    int extra_space_for_hscroll = 0;
-    CWidget *w;
-    WEdit *e;
-
-    if (options & EDITOR_HORIZ_SCROLL)
-	extra_space_for_hscroll = 8;
-
-    wedit = w = CSetupWidget (identifier, parent, x, y,
-			      width + 7, height + 6, C_EDITOR_WIDGET,
-		   ExposureMask | ButtonPressMask | ButtonReleaseMask | \
-		     KeyPressMask | KeyReleaseMask | ButtonMotionMask | \
-			      PropertyChangeMask | StructureNotifyMask | \
-			      EnterWindowMask | LeaveWindowMask, color_palette (option_editor_bg_normal), 1);
-    edit_tri_cursor (w->winid);
-    w->options = options | WIDGET_TAKES_SELECTION;
-
-    w->destroy = edit_destroy_callback;
-    if (filename)
-	w->label = strdup (filename);
-    else
-	w->label = strdup ("");
-
-    if (!made_directory) {
-	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0700);
-	made_directory = 1;
-    }
-    e = w->editor = CMalloc (sizeof (WEdit));
-    if (!w->editor) {
-/* Not essential to translate */
-	CError (_ ("Error initialising editor.\n"));
-	return 0;
-    }
-    w->editor->widget = w;
-    e = w->editor = edit_init (e, height / FONT_PIX_PER_LINE, width / FONT_MEAN_WIDTH, filename, text, starting_directory, text_size);
-    if (!e) {
-	CDestroyWidget (w->ident);
-	return 0;
-    }
-    e->macro_i = -1;
-    e->widget = w;
-
-    set_hint_pos (x + width + 7 + WIDGET_SPACING, y + height + 6 + WIDGET_SPACING + extra_space_for_hscroll);
-    if (extra_space_for_hscroll) {
-	w->hori_scrollbar = CDrawHorizontalScrollbar (catstrs (identifier, ".hsc", 0), parent,
-		x, y + height + 6, width + 6, 12, 0, 0);
-	CSetScrollbarCallback (w->hori_scrollbar->ident, w->ident, link_hscrollbar_to_editor);
-    }
-    if (!(options & EDITOR_NO_TEXT))
-	CDrawText (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING + extra_space_for_hscroll, "%s", e->filename);
-    if (!(options & EDITOR_NO_SCROLL)) {
-	w->vert_scrollbar = CDrawVerticalScrollbar (catstrs (identifier, ".vsc", 0), parent,
-		x + width + 7 + WIDGET_SPACING, y, height + 6, 20, 0, 0);
-	CSetScrollbarCallback (w->vert_scrollbar->ident, w->ident, link_scrollbar_to_editor);
-    }
-    return w;
-}
-
-void update_scroll_bars (WEdit * e)
-{
-    int i, x1, x2;
-    CWidget *scroll;
-    scroll = e->widget->vert_scrollbar;
-    if (scroll) {
-	i = e->total_lines - e->start_line + 1;
-	if (i > e->num_widget_lines)
-	    i = e->num_widget_lines;
-	if (e->total_lines) {
-	    x1 = (double) 65535.0 *e->start_line / (e->total_lines + 1);
-	    x2 = (double) 65535.0 *i / (e->total_lines + 1);
-	} else {
-	    x1 = 0;
-	    x2 = 65535;
-	}
-	if (x1 != scroll->firstline || x2 != scroll->numlines) {
-	    scroll->firstline = x1;
-	    scroll->numlines = x2;
-	    EditExposeRedraw = 1;
-	    render_scrollbar (scroll);
-	    EditExposeRedraw = 0;
-	}
-    }
-    scroll = e->widget->hori_scrollbar;
-    if (scroll) {
-	i = e->max_column - (-e->start_col) + 1;
-	if (i > e->num_widget_columns * FONT_MEAN_WIDTH)
-	    i = e->num_widget_columns * FONT_MEAN_WIDTH;
-	x1 = (double) 65535.0 *(-e->start_col) / (e->max_column + 1);
-	x2 = (double) 65535.0 *i / (e->max_column + 1);
-	if (x1 != scroll->firstline || x2 != scroll->numlines) {
-	    scroll->firstline = x1;
-	    scroll->numlines = x2;
-	    EditExposeRedraw = 1;
-	    render_scrollbar (scroll);
-	    EditExposeRedraw = 0;
-	}
-    }
-}
+#endif
 
 /* returns the position in the edit buffer of a window click */
 long edit_get_click_pos (WEdit * edit, int x, int y)
@@ -243,9 +143,16 @@ static void release_mark (WEdit *edit, XEvent *event)
 	edit_mark_cmd (edit, 0);
     else
 	edit_mark_cmd (edit, 1);
-    if (edit->mark1 != edit->mark2) {
+    if (edit->mark1 != edit->mark2 && event) {
 	edit_get_selection (edit);
-	XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
+#ifdef GTK
+#if 0
+/* *** */
+	gtk_set_selection_owner (CWindowOf (edit->widget));
+#endif
+#else
+	XSetSelectionOwner (CDisplay, XA_PRIMARY, CWindowOf (edit->widget), event->xbutton.time);
+#endif
     }
 }
 
@@ -279,7 +186,227 @@ static void redraw (WEdit *edit, long click)
     mouse_redraw (edit, click);
 }
 
-static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
+void edit_insert_column_of_text (WEdit * edit, unsigned char *data, int size, int width);
+
+/* strips out the first i chars and returns a null terminated string, result must be free'd */
+char *filename_from_url (char *data, int size, int i)
+{
+    char *p, *f;
+    int l;
+    for (p = data + i; (unsigned long) p - (unsigned long) data < size && *p && *p != '\n'; p++);
+    l = (unsigned long) p - (unsigned long) data - i;
+    f = malloc (l + 1);
+    memcpy (f, data + i, l);
+    f[l] = '\0';
+    return f;
+}
+
+static int insert_drop (WEdit * e, Window from, unsigned char *data, int size, int xs, int ys, Atom type, Atom action)
+{
+#ifndef GTK
+    long start_mark = 0, end_mark = 0;
+    int x, y;
+
+    edit_translate_xy (xs, ys, &x, &y);
+/* musn't be able to drop into a block, otherwise a single click will copy a block: */
+    if (eval_marks (e, &start_mark, &end_mark))
+	goto fine;
+    if (start_mark > e->curs1 || e->curs1 >= end_mark)
+	goto fine;
+    if (column_highlighting) {
+	if (!((x >= e->column1 && x < e->column2)
+	      || (x > e->column2 && x <= e->column1)))
+	    goto fine;
+    }
+    return 1;
+  fine:
+    if (from == e->widget->winid && action == CDndClass->XdndActionMove) {
+	edit_block_move_cmd (e);
+	edit_mark_cmd (e, 1);
+	return 0;
+    } else if (from == e->widget->winid) {
+	edit_block_copy_cmd (e);
+	return 0;
+    } else {			/*  data from another widget, or from another application */
+	edit_push_action (e, KEY_PRESS + e->start_display);
+	if (type == XInternAtom (CDisplay, "url/url", False)) {
+	    if (!strncmp ((char *) data, "file:/", 6)) {
+		char *f;
+		edit_insert_file (e, f = filename_from_url ((char *) data, size, strlen ("file:")));
+		free (f);
+	    } else {
+		while (size--)
+		    edit_insert_ahead (e, data[size]);
+	    }
+	} else {
+	    if (column_highlighting) {
+		edit_insert_column_of_text (e, data, size, abs (e->column2 - e->column1));
+	    } else {
+		while (size--)
+		    edit_insert_ahead (e, data[size]);
+	    }
+	}
+    }
+    CExpose (e->widget->ident);
+#endif
+    return 0;
+}
+
+static char *mime_majors[2] =
+    {"text", 0};
+
+struct mouse_funcs edit_mouse_funcs =
+{
+    0,
+    (void (*)(int, int, int *, int *)) xy,
+    (long (*)(void *, int, int)) cp,
+    (int (*)(void *, long *, long *)) marks,
+    (int (*)(void *, long, long, long)) erange,
+    (void (*)(void *)) fin_mark,
+    (void (*)(void *)) move_mark,
+    (void (*)(void *, XEvent *)) release_mark,
+    (char *(*)(void *, long, long, int *, int *)) get_block,
+    (void (*)(void *, long, int)) move,
+    0,
+    (void (*)(void *, XEvent *)) dclick,
+    (void (*)(void *, long)) redraw,
+    (int (*)(void *, Window, unsigned char *, int, int, int, Atom, Atom)) insert_drop,
+    (void (*)(void *)) edit_block_delete,
+    DndText,
+    mime_majors
+};
+
+#ifndef GTK
+
+extern int option_editor_bg_normal;
+void edit_tri_cursor (Window win);
+
+/* starting_directory is for the filebrowser */
+CWidget *CDrawEditor (const char *identifier, Window parent, int x, int y,
+	   int width, int height, const char *text, const char *filename,
+		const char *starting_directory, unsigned int options, unsigned long text_size)
+{
+    static made_directory = 0;
+    int extra_space_for_hscroll = 0;
+    CWidget *w;
+    WEdit *e;
+
+    if (options & EDITOR_HORIZ_SCROLL)
+	extra_space_for_hscroll = 8;
+
+    wedit = w = CSetupWidget (identifier, parent, x, y,
+			      width + 7, height + 6, C_EDITOR_WIDGET,
+		   ExposureMask | ButtonPressMask | ButtonReleaseMask | \
+		     KeyPressMask | KeyReleaseMask | ButtonMotionMask | \
+			      PropertyChangeMask | StructureNotifyMask | \
+			      EnterWindowMask | LeaveWindowMask, color_palette (option_editor_bg_normal), 1);
+
+    xdnd_set_dnd_aware (CDndClass, w->winid, 0);
+    xdnd_set_type_list (CDndClass, w->winid, xdnd_typelist_send[DndText]);
+
+    edit_tri_cursor (w->winid);
+    w->options = options | WIDGET_TAKES_SELECTION;
+
+    w->destroy = edit_destroy_callback;
+    if (filename)
+	w->label = strdup (filename);
+    else
+	w->label = strdup ("");
+
+    if (!made_directory) {
+	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0700);
+	made_directory = 1;
+    }
+    e = w->editor = CMalloc (sizeof (WEdit));
+    w->funcs = mouse_funcs_new (w->editor, &edit_mouse_funcs);
+
+    if (!w->editor) {
+/* Not essential to translate */
+	CError (_ ("Error initialising editor.\n"));
+	return 0;
+    }
+    w->editor->widget = w;
+    w->editor = edit_init (e, height / FONT_PIX_PER_LINE, width / FONT_MEAN_WIDTH, filename, text, starting_directory, text_size);
+    w->funcs->data = (void *) w->editor;
+    if (!w->editor) {
+	free (e);
+	CDestroyWidget (w->ident);
+	return 0;
+    }
+    e->macro_i = -1;
+    e->widget = w;
+
+    set_hint_pos (x + width + 7 + WIDGET_SPACING, y + height + 6 + WIDGET_SPACING + extra_space_for_hscroll);
+    if (extra_space_for_hscroll) {
+	w->hori_scrollbar = CDrawHorizontalScrollbar (catstrs (identifier, ".hsc", 0), parent,
+		x, y + height + 6, width + 6, 12, 0, 0);
+	CSetScrollbarCallback (w->hori_scrollbar->ident, w->ident, link_hscrollbar_to_editor);
+    }
+    if (!(options & EDITOR_NO_TEXT))
+	CDrawText (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING + extra_space_for_hscroll, "%s", e->filename);
+    if (!(options & EDITOR_NO_SCROLL)) {
+	w->vert_scrollbar = CDrawVerticalScrollbar (catstrs (identifier, ".vsc", 0), parent,
+		x + width + 7 + WIDGET_SPACING, y, height + 6, 20, 0, 0);
+	CSetScrollbarCallback (w->vert_scrollbar->ident, w->ident, link_scrollbar_to_editor);
+    }
+    return w;
+}
+
+#endif
+
+#ifdef GTK
+
+void update_scroll_bars (WEdit * e)
+{
+
+}
+
+#else
+
+void update_scroll_bars (WEdit * e)
+{
+    int i, x1, x2;
+    CWidget *scroll;
+    scroll = e->widget->vert_scrollbar;
+    if (scroll) {
+	i = e->total_lines - e->start_line + 1;
+	if (i > e->num_widget_lines)
+	    i = e->num_widget_lines;
+	if (e->total_lines) {
+	    x1 = (double) 65535.0 *e->start_line / (e->total_lines + 1);
+	    x2 = (double) 65535.0 *i / (e->total_lines + 1);
+	} else {
+	    x1 = 0;
+	    x2 = 65535;
+	}
+	if (x1 != scroll->firstline || x2 != scroll->numlines) {
+	    scroll->firstline = x1;
+	    scroll->numlines = x2;
+	    EditExposeRedraw = 1;
+	    render_scrollbar (scroll);
+	    EditExposeRedraw = 0;
+	}
+    }
+    scroll = e->widget->hori_scrollbar;
+    if (scroll) {
+	i = e->max_column - (-e->start_col) + 1;
+	if (i > e->num_widget_columns * FONT_MEAN_WIDTH)
+	    i = e->num_widget_columns * FONT_MEAN_WIDTH;
+	x1 = (double) 65535.0 *(-e->start_col) / (e->max_column + 1);
+	x2 = (double) 65535.0 *i / (e->max_column + 1);
+	if (x1 != scroll->firstline || x2 != scroll->numlines) {
+	    scroll->firstline = x1;
+	    scroll->numlines = x2;
+	    EditExposeRedraw = 1;
+	    render_scrollbar (scroll);
+	    EditExposeRedraw = 0;
+	}
+    }
+}
+
+#endif
+
+void edit_mouse_mark (WEdit * edit, XEvent * event, int double_click)
 {
     edit_update_curs_row (edit);
     edit_update_curs_col (edit);
@@ -293,24 +420,16 @@ static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
 	edit->found_len = 0;
     }
     mouse_mark (
-	(void  *) edit,
 	event,
-	ce,
-	(void (*) (int, int, int *, int *)) xy,
-	(long (*) (void *, int, int)) cp,
-	(int (*) (void *, long *, long *)) marks,
-	(int (*) (void *, long, long, long)) erange,
-	(void (*) (void *)) fin_mark,
-	(void (*) (void *)) move_mark,
-	(void (*) (void *, XEvent *)) release_mark,
-	(char * (*) (void *, long, long, int *, int *)) get_block,
-	(void (*) (void *, long, int)) move,
-	0,
-	(void (*) (void *, XEvent *)) dclick,
-	(void (*) (void *, long)) redraw
+	double_click,
+	edit->widget->funcs
     );
 }
 
+#ifdef GTK
+
+#else
+
 void link_scrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
 {
     int i, start_line;
@@ -362,6 +481,12 @@ void link_scrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent * x
     }
 }
 
+#endif
+
+#ifdef GTK
+
+#else
+
 void link_hscrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
 {
     int i, start_col;
@@ -411,6 +536,8 @@ void link_hscrollbar_to_editor (CWidget * scrollbar, CWidget * editor, XEvent *
     }
 }
 
+#endif
+
 /* 
    This section comes from rxvt-2.21b1/src/screen.c by
    Robert Nation <nation@rocket.sanders.lockheed.com> &
@@ -480,6 +607,19 @@ void paste_prop (void *data, void (*insert) (void *, int), Window win, unsigned
 	int actual_fmt, i;
 	unsigned long nitems;
 
+#ifdef GTK
+#if 0
+/* *** */
+	if (gtk_get_window_property (win, prop,
+				nread / 4, 65536, delete,
+			      AnyPropertyType, &actual_type, &actual_fmt,
+				&nitems, &bytes_after,
+				&s) != Success) {
+	    XFree (s);
+	    return;
+	}
+#endif
+#else
 	if (XGetWindowProperty (CDisplay, win, prop,
 				nread / 4, 65536, delete,
 			      AnyPropertyType, &actual_type, &actual_fmt,
@@ -488,6 +628,7 @@ void paste_prop (void *data, void (*insert) (void *, int), Window win, unsigned
 	    XFree (s);
 	    return;
 	}
+#endif
 	nread += nitems;
 	for (i = 0; i < nitems; i++)
 	    (*insert) (data, s[i]);
@@ -533,8 +674,19 @@ void edit_update_screen (WEdit * e)
 /* pop all events for this window for internal handling */
     if (e->force & (REDRAW_CHAR_ONLY | REDRAW_COMPLETELY)) {
 	edit_render_keypress (e);
+#ifdef GTK
+    } else if (
+#if 0
+/* *** */
+    gtk_edit_key_pending () || gtk_edit_mouse_pending ()
+#else
+    0
+#endif
+    ) {
+#else
     } else if (CCheckWindowEvent (e->widget->winid, ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, 0)
 	       || CKeyPending ()) {
+#endif
 	e->force |= REDRAW_PAGE;
 	return;
     } else {
@@ -544,51 +696,14 @@ void edit_update_screen (WEdit * e)
 
 extern int space_width;
 
-static void edit_insert_column_of_text (WEdit * edit, unsigned char *data, int size, int width)
-{
-    long cursor;
-    int i, col;
-    cursor = edit->curs1;
-    col = edit_get_col (edit);
-    for (i = 0; i < size; i++) {
-	if (data[i] == '\n') {	/* fill in and move to next line */
-	    int l;
-	    long p;
-	    if (edit_get_byte (edit, edit->curs1) != '\n') {
-		l = width - (edit_get_col (edit) - col);
-		while (l > 0) {
-		    edit_insert (edit, ' ');
-		    l -= space_width;
-		}
-	    }
-	    for (p = edit->curs1;; p++) {
-		if (p == edit->last_byte)
-		    edit_insert_ahead (edit, '\n');
-		if (edit_get_byte (edit, p) == '\n') {
-		    p++;
-		    break;
-		}
-	    }
-	    edit_cursor_move (edit, edit_move_forward3 (edit, p, col, 0) - edit->curs1);
-	    l = col - edit_get_col (edit);
-	    while (l >= space_width) {
-		edit_insert (edit, ' ');
-		l -= space_width;
-	    }
-	    continue;
-	}
-	edit_insert (edit, data[i]);
-    }
-    edit_cursor_move (edit, cursor - edit->curs1);
-}
-
+#ifdef HAVE_DND
 #define free_data if (data) {free(data);data=0;}
 
 /* handles drag and drop */
-void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
+void handle_client_message (CWidget * w, XEvent * xevent)
 {
     int data_type;
-    unsigned char *data;
+    unsigned char *data = 0;
     unsigned long size;
     int xs, ys;
     long start_line;
@@ -608,7 +723,7 @@ void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
     }
     data_type = CGetDrop (xevent, &data, &size, &xs, &ys);
 
-    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height) {
+    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= CWidthOf (w) || ys >= CHeightOf (w)) {
 	free_data;
 	return;
     }
@@ -683,6 +798,9 @@ void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
     e->force |= REDRAW_COMPLETELY | REDRAW_LINE;
     free_data;
 }
+#endif
+
+#ifndef GTK
 
 int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 {
@@ -712,23 +830,36 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	selection_send (&(xevent->xselectionrequest));
 	return 1;
 /*  case SelectionClear:   ---> This is handled by coolnext.c: CNextEvent() */
+#ifdef HAVE_DND
     case ClientMessage:
-	handle_client_message (w, xevent, cwevent);
+	handle_client_message (w, xevent);
 	r = 1;
+#endif
 	break;
     case ButtonPress:
 	CFocus (w);
 	edit_render_tidbits (w);
     case ButtonRelease:
-	if (xevent->xbutton.state & ControlMask)
+	if (xevent->xbutton.state & ControlMask) {
+	    if (!column_highlighting)
+		edit_push_action (e, COLUMN_OFF);
 	    column_highlighting = 1;
-	else
+	} else {
+	    if (column_highlighting)
+		edit_push_action (e, COLUMN_ON);
 	    column_highlighting = 0;
+	}
     case MotionNotify:
 	if (!xevent->xmotion.state && xevent->type == MotionNotify)
 	    return 0;
 	resolve_button (xevent, cwevent);
-	edit_mouse_mark (e, xevent, cwevent);
+	if ((cwevent->button == Button4 || cwevent->button == Button5)
+	    && (xevent->type == ButtonRelease)) {
+	    /* ahaack: wheel mouse mapped as button 4 and 5 */
+	    r = edit_execute_key_command (e, (cwevent->button == Button5) ? CK_Page_Down : CK_Page_Up, -1);
+	    break;
+	}
+	edit_mouse_mark (e, xevent, cwevent->double_click);
 	break;
     case Expose:
 	edit_render_expose (e, &(xevent->xexpose));
@@ -742,11 +873,13 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	return 1;
 	break;
     case KeyRelease:
+#if 0
 	if (column_highlighting) {
 	    column_highlighting = 0;
 	    e->force = REDRAW_COMPLETELY | REDRAW_LINE;
 	    edit_mark_cmd (e, 1);
 	}
+#endif
 	break;
     case KeyPress:
 	cwevent->ident = w->ident;
@@ -770,7 +903,9 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	cwevent->ident = w->ident;
 	cwevent->command = xevent->xkey.keycode;
 	r = cwevent->handled = edit_execute_key_command (e, xevent->xkey.keycode, -1);
-	break;
+	if (r)
+	    edit_update_screen (e);
+	return r;
     default:
 	return 0;
     }
@@ -778,6 +913,8 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
     return r;
 }
 
+#endif	/* ! GTK */
+
 #else
 
 WEdit *wedit;
