@@ -12,6 +12,7 @@
 #include <string.h>
 #include <stdlib.h>		/* atoi */
 #include "fs.h"
+#include "mad.h"
 #include "x.h"
 #include "dir.h"
 #include "panel.h"
@@ -490,6 +491,12 @@ panel_action_view_unfiltered (GtkWidget *widget, WPanel *panel)
 	view_simple_cmd (panel);
 }
 
+void
+panel_action_edit (GtkWidget *widget, WPanel *panel)
+{
+	edit_cmd (panel);
+}
+
 void
 panel_action_properties (GtkWidget *widget, WPanel *panel)
 {
@@ -536,7 +543,7 @@ typedef struct {
  */
 common_menu_t common_panel_actions [] = {
 	{ N_("Copy..."),         (context_menu_callback) copy_cmd },
-	{ N_("Rename/move.."),   (context_menu_callback) ren_cmd },
+	{ N_("Rename/move..."),  (context_menu_callback) ren_cmd },
 	{ N_("Delete..."),       (context_menu_callback) delete_cmd },
 	{ NULL, NULL }
 };
@@ -1003,7 +1010,7 @@ panel_make_local_copies_and_transfer (GtkWidget *widget, GdkEventDragRequest *ev
 			p += strlen (p) + 1;
 		}
 	} else {
-		filename = concat_dir_and_file (panel->cwd, panel->dir.list [i].fname);
+		filename = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 		localname = mc_getlocalcopy (filename);
 		free (filename);
 		*data = localname;
@@ -1206,7 +1213,7 @@ panel_create_file_list (WPanel *panel)
 	file_list = gtk_blist_new_with_titles (items, titles);
 	clist = GTK_CLIST (file_list);
 	panel_configure_file_list (panel, file_list);
-	free (titles);
+	g_free (titles);
 
 	gtk_signal_connect_after (GTK_OBJECT (file_list), "size_allocate",
 				  GTK_SIGNAL_FUNC (panel_file_list_size_allocate_hook),
@@ -1389,7 +1396,13 @@ panel_create_icon_display (WPanel *panel)
 
 	icon_field = GNOME_ICON_LIST (gnome_icon_list_new ());
 	gnome_icon_list_set_separators (icon_field, " /-_.");
+	gnome_icon_list_set_row_spacing (icon_field, 2);
+	gnome_icon_list_set_col_spacing (icon_field, 2);
+	gnome_icon_list_set_icon_border (icon_field, 4);
+	gnome_icon_list_set_text_spacing (icon_field, 2);
 	
+	icon_field->desired_text_width = 100;
+		
 	gnome_icon_list_set_selection_mode (icon_field, GTK_SELECTION_MULTIPLE);
 
 	gtk_signal_connect (GTK_OBJECT (icon_field), "select_icon",
@@ -1601,7 +1614,7 @@ display_mini_info (WPanel *panel)
 	if (panel->searching){
 		char *str = copy_strings (N_("Search: "), panel->search_buffer, NULL);
 		
-		gtk_clip_label_set (label, str);
+		gtk_label_set (label, str);
 		free (str);
 		return;
 	}
@@ -1613,7 +1626,7 @@ display_mini_info (WPanel *panel)
 			 size_trunc_sep (panel->total), panel->marked,
 			 panel->marked == 1 ? "" : "s");
 		
-		gtk_clip_label_set (label, buffer);
+		gtk_label_set (label, buffer);
 		return;
 	}
 
@@ -1630,10 +1643,10 @@ display_mini_info (WPanel *panel)
 
 			link_target [len] = 0;
 			str = copy_strings ("-> ", link_target, NULL);
-			gtk_clip_label_set (label, str);
+			gtk_label_set (label, str);
 			free (str);
 		} else 
-			gtk_clip_label_set (label, N_("<readlink failed>"));
+			gtk_label_set (label, N_("<readlink failed>"));
 		return;
 	}
 
@@ -1644,12 +1657,12 @@ display_mini_info (WPanel *panel)
 		buffer = xmalloc (len + 2, "display_mini_info");
 		format_file (buffer, panel, panel->selected, panel->estimated_total-2, 0, 1);
 		buffer [len] = 0;
-		gtk_clip_label_set (label, buffer);
+		gtk_label_set (label, buffer);
 		free (buffer);
 	}
 	if (panel->list_type == list_icons){
 		if (panel->marked == 0){
-			gtk_clip_label_set (label, "");
+			gtk_label_set (label, "");
 		}
 	}
 }
@@ -1809,7 +1822,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	panel_update_marks (panel);
 	
 	/* ministatus */
-	panel->ministatus = gtk_clip_label_new ("");
+	panel->ministatus = gtk_label_new (""); /* was a cliplabel */
 	gtk_misc_set_alignment (GTK_MISC (panel->ministatus), 0.0, 0.0);
 	gtk_misc_set_padding (GTK_MISC (panel->ministatus), 3, 0);
 	gtk_widget_show (panel->ministatus);
@@ -1835,7 +1848,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 	gtk_container_border_width (GTK_CONTAINER (frame), 3);
 
-	panel->status = gtk_clip_label_new ("");
+	panel->status = gtk_label_new (""); /* used to be a cliplabel */
 	gtk_misc_set_alignment (GTK_MISC (panel->status), 0.0, 0.5);
 	gtk_misc_set_padding (GTK_MISC (panel->status), 3, 0);
 	gtk_container_add (GTK_CONTAINER (frame), panel->status);
