@@ -272,6 +272,18 @@ void view_cmd (WPanel *panel)
     do_view_cmd (panel, 0);
 }
 
+void view_file_cmd (WPanel *panel)
+{
+    char *filename;
+
+    panel = get_a_panel (panel);
+    filename = input_dialog (_(" View file "), _(" Filename:"), selection (panel)->fname);
+    if (!filename) return;
+
+    view_file (filename, 0, use_internal_view);
+    free (filename);
+}
+
 void view_simple_cmd (WPanel *panel)
 {
     do_view_cmd (panel, 1);
@@ -1259,13 +1271,13 @@ static void nice_cd (char *text, char *xtext, char *help, char *prefix, int to_h
 void netlink_cmd (void)
 {
     nice_cd (_(" Link to a remote machine "), _(machine_str),
-	     "[Network File System]", "mc:", 1);
+	     "[Network File System]", "/#mc:", 1);
 }
 
 void ftplink_cmd (void)
 {
     nice_cd (_(" FTP to machine "), _(machine_str),
-	     "[FTP File System]", "ftp://", 1);
+	     "[FTP File System]", "/#ftp:", 1);
 }
 
 #ifdef HAVE_SETSOCKOPT
@@ -1294,9 +1306,9 @@ void source_routing (void)
 void undelete_cmd (void)
 {
     nice_cd (_(" Undelete files on an ext2 file system "),
-	     _(" Enter the file system name where you want to run the\n "
-	       " undelete file system on: (F1 for details)"),
-	     "[Undelete File System]", "undel:", 0);
+	     _(" Enter device (without /dev/) to undelete\n "
+	       "  files on: (F1 for details)"),
+	     "[Undelete File System]", "/#undel:", 0);
 }
 #endif
 
