@@ -71,6 +71,7 @@
 #include "complete.h"
 #include "main.h"
 #include "key.h"		/* XCTRL and ALT macros */
+#include "../vfs/vfs.h"
 
 /* This flag is used in filename_completion_function */
 int ignore_filenames = 0;
@@ -137,13 +138,13 @@ char *filename_completion_function (char *text, int state)
 	       and `command`.
 	       Maybe a dream - UNIMPLEMENTED yet. */
         }
-        directory = opendir (dirname);
+        directory = mc_opendir (dirname);
         filename_len = strlen (filename);
     }
 
     /* Now that we have some state, we can read the directory. */
 
-    while (directory && (entry = readdir (directory))){
+    while (directory && (entry = mc_readdir (directory))){
         /* Special case for no filename.
 	   All entries except "." and ".." match. */
         if (!filename_len){
@@ -167,7 +168,7 @@ char *filename_completion_function (char *text, int state)
 	    strcat (tmp, entry->d_name);
 	    canonicalize_pathname (tmp);
 	    /* Unix version */
-	    if (!stat (tmp, &tempstat)){
+	    if (!mc_stat (tmp, &tempstat)){
 	    	uid_t my_uid = getuid ();
 	    	gid_t my_gid = getgid ();
 	    	
@@ -198,7 +199,7 @@ char *filename_completion_function (char *text, int state)
 
     if (!entry){
         if (directory){
-	    closedir (directory);
+	    mc_closedir (directory);
 	    directory = NULL;
 	}
         if (dirname){
@@ -978,6 +979,8 @@ int complete_engine (WInput *in, int what_to_do)
     	    if (insert_text (in, in->completions [0], strlen (in->completions [0]))){
     	        if (in->completions [1])
     	    	    beep ();
+		else
+		    free_completions (in);
 	    } else
 	        beep ();
         }
