@@ -6,7 +6,7 @@
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
-#ifndef _OS_NT
+#ifndef OS2_NT
 #include <sys/param.h>
 #endif
 #include <sys/stat.h>
@@ -19,7 +19,11 @@
 
 /* unistd.h defines _POSIX_VERSION on POSIX.1 systems. */
 #if defined(HAVE_DIRENT_H) || defined(_POSIX_VERSION)
+# ifdef __os2__
+#   include "dirent.h"
+# else
 #   include <dirent.h>
+# endif
 #   define NLENGTH(dirent) (strlen ((dirent)->d_name))
 #   define DIRENT_LENGTH_COMPUTED 1
 #else
