@@ -260,6 +260,8 @@ static void fill_listbox (void)
 		else
 		    listbox_add_item (l_hotlist, 0, 0, current->label, current);
 	    break;
+	default:
+	    break;
 	}
 	current = current->next;
     }
@@ -1382,9 +1384,12 @@ void load_hotlist (void)
 	int	result;
 	char    *msg;
 
-	msg = copy_strings (_("Hotlist is now kept in file ~/"), HOTLIST_FILENAME,
-			    _("MC will load hotlist from ~/"),  PROFILE_NAME,
-			    _(" and then delete [Hotlist] section there"), NULL);
+	msg = copy_strings (_("Hotlist is now kept in file ~/"),
+			    HOTLIST_FILENAME, "\n",
+			    _("MC will load hotlist from ~/"),
+			    PROFILE_NAME, "\n",
+			    _("and then delete [Hotlist] section there"),
+			    NULL);
 	message (0, _(" Hotlist Load "), msg);
 	free (msg);
 	
@@ -1605,6 +1610,8 @@ void done_hotlist (void)
     hotlist = current_group = 0;
     l_hotlist = 0;
     current_group = 0;
+    if (tkn_buf)
+        free (tkn_buf);
 }
 
 
