@@ -75,7 +75,7 @@
 
 #include "x.h"
 
-/* "$Id: layout.c,v 1.8 1998/05/20 02:20:09 unammx Exp $" */
+/* "$Id: layout.c,v 1.9 1998/09/08 12:13:00 proskin Exp $" */
 
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
@@ -843,7 +843,8 @@ void set_hintbar(char *str)
 {
 #ifndef HAVE_X
     if (xterm_flag && xterm_hintbar) {
-        fprintf (stderr, "\33]0;mc - %s\7", str);
+	fprintf (stdout, "\33]0;mc - %s\7", str);
+	fflush (stdout);
     } else
 #endif
     {
