@@ -134,7 +134,7 @@
 
 #include "popt.h"
 
-/* "$Id: main.c,v 1.37 1998/05/20 02:20:12 unammx Exp $" */
+/* "$Id: main.c,v 1.48 1998/09/22 14:45:38 proskin Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -370,40 +370,6 @@ try_to_select (WPanel *panel, char *name)
     display_mini_info (panel);
 }
 
-/*
- * cd_try_to_select:
- *
- *  If we moved to the parent directory move the selection pointer to
- *  the old directory name
- */
-void
-cd_try_to_select (WPanel *panel)
-{
-    char *p, *q;
-    int i, j = 4;
-
-    if (strlen (panel->lwd) > strlen (panel->cwd)
-	&& strncmp (panel->cwd, panel->lwd, strlen (panel->cwd)) == 0
-	&& strchr (panel->lwd + strlen (panel->cwd) + 1, PATH_SEP) == 0)
-	try_to_select (panel, panel->lwd);
-    else
-#ifdef USE_VFS
-	if ((!strncmp (panel->lwd, "tar:", 4) && 
-             !strncmp (panel->lwd + 4, panel->cwd, strlen (panel->cwd))) ||
-             ((i = extfs_prefix_to_type (panel->lwd)) != -1 && 
-             !strncmp (panel->lwd + (j = strlen (extfs_get_prefix (i)) + 1), 
-             panel->cwd, strlen (panel->cwd)))) {
-        p = strdup (panel->lwd + j + strlen (panel->cwd));
-        q = strchr (p, PATH_SEP);
-        if (q != NULL && (q != p || (q = strchr (q + 1, PATH_SEP)) != NULL))
-            *q = 0;
-        try_to_select (panel, p);
-        free (p);
-    } else
-#endif
-	try_to_select (panel, NULL);
-}
-
 void
 reload_panelized (WPanel *panel)
 {
@@ -940,18 +906,56 @@ directory_history_add (WPanel * panel, char *s)
     panel_update_marks (panel);
 }
 
+/* Translate ftp://user:password@host/directory to 
+              #ftp:user:password@host/directory.
+ */
+static char *
+translate_url_to_new_syntax (const char *p)
+{
+    if (strncmp (p, "ftp://", 6) == 0)
+        return copy_strings ("#ftp:", p + 6, 0);
+    else
+        return strdup (p);
+}
+
+/*
+ *  If we moved to the parent directory move the selection pointer to
+ *  the old directory name; If we leave VFS dir, remove FS specificator.
+ *  Warn: This code spoils lwd string.
+ */
+char*
+get_parent_dir_name (char *cwd, char *lwd)
+{
+    char *p;
+    if (strlen (lwd) > strlen (cwd))
+	if((p=strrchr(lwd, PATH_SEP)) && !strncmp(cwd, lwd, p-lwd)){
+#ifdef USE_VFS
+	    char *q;
+	    if((q=strrchr(p, '#')) != 0){
+		/* Here we need proper VFS function */
+		if(!strcmp((q+1), "utar") || extfs_which(q+1) >= 0 || sfs_which(q+1) >= 0)
+		    *q = '\0';
+	    }
+#endif /* USE_VFS */
+	return (p+1);
+	}
+    return NULL;
+}
+
 /* Changes the current panel directory */
 int
 _do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
 {
     char *directory, *olddir;
     char temp [MC_MAXPATHLEN];
+    char *translated_url;
 #ifdef USE_VFS    
     vfs *oldvfs;
     vfsid oldvfsid;
     struct vfs_stamping *parent;
 #endif    
     olddir = strdup (panel->cwd);
+    translated_url = new_dir = translate_url_to_new_syntax (new_dir);
 
     /* Convert *new_path to a suitable pathname, handle ~user */
     
@@ -965,12 +969,14 @@ _do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
 	}
     }
     directory = *new_dir ? new_dir : home_dir;
-
+    
     if (mc_chdir (directory) == -1){
 	strcpy (panel->cwd, olddir);
 	free (olddir);
+        free (translated_url);
 	return 0;
     }
+    free (translated_url);
 
     /* Success: save previous directory, shutdown status of previous dir */
     strcpy (panel->lwd, olddir);
@@ -984,7 +990,6 @@ _do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
     vfs_add_noncurrent_stamps (oldvfs, oldvfsid, parent);
     vfs_rm_parents (parent);
 #endif
-    free (olddir);
 
     subshell_chdir (panel->cwd);
 
@@ -998,9 +1003,12 @@ _do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
     panel->dirs_marked = 0;
     panel->total = 0;
     panel->searching = 0;
-    cd_try_to_select (panel);
+    try_to_select (panel, get_parent_dir_name(panel->cwd, olddir));
     load_hint ();
     panel_update_contents (panel);
+    
+    free (olddir);
+
     return 1;
 }
 
@@ -1244,6 +1252,7 @@ static menu_entry RightMenu [] = {
 static menu_entry FileMenu [] = {
     { ' ', N_("&User menu          F2"), 'U', user_menu_cmd },
     { ' ', N_("&View               F3"), 'V', view_cmd },
+    { ' ', N_("Vie&w file...         "), 'W', view_file_cmd },
     { ' ', N_("&Filtered view     M-!"), 'F', filtered_view_cmd },
     { ' ', N_("&Edit               F4"), 'E', edit_cmd },
     { ' ', N_("&Copy               F5"), 'C', copy_cmd },
@@ -1333,19 +1342,19 @@ init_menu (void)
     int i;
 
 #ifdef HAVE_X    
-    MenuBar [0] = create_menu (_(" Left "), PanelMenu, menu_entries (PanelMenu));
+    MenuBar [0] = create_menu (_(" &Left "), PanelMenu, menu_entries (PanelMenu));
 #else
-    MenuBar [0] = create_menu ( horizontal_split ? _(" Above ") : _(" Left "), 
+    MenuBar [0] = create_menu ( horizontal_split ? _(" &Above ") : _(" &Left "), 
                                 PanelMenu, menu_entries (PanelMenu));
 #endif
-    MenuBar [1] = create_menu (_(" File "), FileMenu, menu_entries (FileMenu));
-    MenuBar [2] = create_menu (_(" Command "), CmdMenu, menu_entries (CmdMenu));
-    MenuBar [3] = create_menu (_(" Options "), OptMenu, menu_entries (OptMenu));
+    MenuBar [1] = create_menu (_(" &File "), FileMenu, menu_entries (FileMenu));
+    MenuBar [2] = create_menu (_(" &Command "), CmdMenu, menu_entries (CmdMenu));
+    MenuBar [3] = create_menu (_(" &Options "), OptMenu, menu_entries (OptMenu));
 #ifndef HAVE_XVIEW
 #ifdef HAVE_X
-    MenuBar [4] = create_menu (_(" Right "), RightMenu, menu_entries (PanelMenu));
+    MenuBar [4] = create_menu (_(" &Right "), RightMenu, menu_entries (PanelMenu));
 #else
-    MenuBar [4] = create_menu (horizontal_split ? _(" Below ") : _(" Right "), 
+    MenuBar [4] = create_menu (horizontal_split ? _(" &Below ") : _(" &Right "), 
 			       RightMenu, menu_entries (PanelMenu));
     for (i = 0; i < 5; i++)
 	MenuBarEmpty [i] = create_menu (MenuBar [i]->name, 0, 0);
@@ -1451,6 +1460,20 @@ toggle_align_extensions (void)
     align_extensions = !align_extensions;
 }
 
+/*
+ * Just a hack for allowing url-like pathnames to be accepted from the
+ * command line.
+ */
+static void
+translated_mc_chdir (char *dir)
+{
+	char *newdir;
+
+	newdir = translate_url_to_new_syntax (dir);
+	mc_chdir (newdir);
+	free (newdir);
+}
+
 #ifndef PORT_HAS_CREATE_PANELS
 void
 create_panels (void)
@@ -1483,15 +1506,15 @@ create_panels (void)
 	     */
 	    mc_get_current_wd (original_dir, sizeof (original_dir)-2);
 	}
-	mc_chdir (this_dir);
+	translated_mc_chdir (this_dir);
     }
     set_display_type (current_index, current_mode);
 
     /* The other panel */
     if (other_dir){
 	if (original_dir [0])
-	    mc_chdir (original_dir);
-	mc_chdir (other_dir);
+	    translated_mc_chdir (original_dir);
+	translated_mc_chdir (other_dir);
     }
     set_display_type (other_index, other_mode);
 
@@ -1776,20 +1799,16 @@ static key_map default_map [] = {
 #ifndef HAVE_X
 static void setup_sigwinch ()
 {
-#ifndef OS2_NT
+#if (defined(HAVE_SLANG) || (NCURSES_VERSION_MAJOR >= 4)) && \
+   !defined(OS2_NT) && defined(SIGWINCH)
     struct sigaction act, oact;
-    
-#   if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#       ifdef SIGWINCH
-            act.sa_handler = flag_winch;
-            sigemptyset (&act.sa_mask);
-	    act.sa_flags = 0;
-#           ifdef SA_RESTART
-                act.sa_flags |= SA_RESTART;
-#           endif
-            sigaction (SIGWINCH, &act, &oact);
-#       endif
+    act.sa_handler = flag_winch;
+    sigemptyset (&act.sa_mask);
+    act.sa_flags = 0;
+#   ifdef SA_RESTART
+	act.sa_flags |= SA_RESTART;
 #   endif
+    sigaction (SIGWINCH, &act, &oact);
 #endif
 }
 
@@ -2335,6 +2354,8 @@ void
 OS_Setup ()
 {
     char   *termvalue;
+    char   *mc_libdir;
+	
     termvalue = getenv ("TERM");
     if (!termvalue){
 	fprintf (stderr, _("The TERM environment variable is unset!\n"));
@@ -2358,8 +2379,13 @@ OS_Setup ()
     sprintf (control_file, CONTROL_FILE, getpid ());
     my_putenv ("MC_CONTROL_FILE", control_file);
     
-    /* This is the directory, where MC was installed, on Unix this always is LIBDIR */
-    mc_home = strdup(LIBDIR);
+    /* This is the directory, where MC was installed, on Unix this is LIBDIR */
+    /* and can be overriden by the MC_LIBDIR environment variable */
+    if ((mc_libdir = getenv ("MC_LIBDIR")) != NULL) {
+	mc_home = strdup(mc_libdir);
+    } else {
+	mc_home = strdup(LIBDIR);
+    }
 }
 
 static void
@@ -2663,7 +2689,7 @@ static struct argp_option argp_options [] = {
     { "color",          'c', NULL, 0, N_("Force color mode."), 0 },
     { "colors", 	'C', "COLORS", 0, N_("Specify colors (use --help-colors to get a list)."), 1 },
 #ifdef HAVE_SUBSHELL_SUPPORT
-    { "dbgsubshell", 	'X', NULL, 0, N_("[DEVEL-ONLY: Debug the subshell."), 0 },
+    { "dbgsubshell", 	'X', NULL, 0, N_("[DEVEL-ONLY: Debug the subshell]"), 0 },
 #endif
     { "edit", 		'e', "EDIT", 0, N_("Startup the internal editor."), 1 },
     { "help", 		'h', NULL, 0, N_("Shows this help message."), 0 },
@@ -2691,7 +2717,7 @@ static struct argp_option argp_options [] = {
 #if defined(HAVE_SLANG) && !defined(OS2_NT)
     { "termcap", 	't', NULL, 0, N_("Activate support for the TERMCAP variable."), 0 },
 #endif
-    { "version", 	'V', NULL, 0, N_("Report versionand configuration options."), 0 },
+    { "version", 	'V', NULL, 0, N_("Report version and configuration options."), 0 },
     { "view", 		'v', NULL, 0, N_("Start up into the viewer mode."), 0 },
     { "xterm", 		'x', NULL, 0, N_("Force xterm mouse support and screen save/restore"), 0 },
     { "geometry",       GEOMETRY_KEY,  "GEOMETRY", 0, N_("Geometry for the window"), 0 },
@@ -2956,18 +2982,17 @@ int main (int argc, char *argv [])
     bindtextdomain ("mc", LOCALEDIR);
     textdomain ("mc");
 
-    vfs_init ();
-
     /* Initialize list of all user group for timur_clr_mode */
     init_groups ();
     
     OS_Setup ();
 
+    vfs_init ();
+
 #ifdef HAVE_X
     /* NOTE: This call has to be before any our argument handling :) */
 
 #ifdef HAVE_GNOME
-    session_management_setup (argv [0]);
     {
 	char *base = x_basename (argv [0]);
 
@@ -2980,6 +3005,7 @@ int main (int argc, char *argv [])
 	}
     }
     gnome_init ("gmc", &mc_argp_parser, argc, argv, 0, NULL);
+    session_management_setup (argv [0]);
     probably_finish_program ();
 #endif
     
