@@ -30,7 +30,7 @@ void do_execute (const char *shell, const char *command, int internal_command);
 #define execute_internal(command,args) do_execute (command, args, 1)
 
 /* Execute functions that use the shell to execute */
-void shell_execute (char *command, int internal_command);
+void shell_execute (char *command, int flags);
 void execute (char *command);
 
 /* This one executes a shell */
@@ -58,6 +58,7 @@ extern int option_backup_ext_int;
 extern int option_auto_para_formatting;
 extern int option_typewriter_wrap;
 extern int edit_confirm_save;
+extern int option_syntax_highlighting;
 #endif	/* ! USE_INTERNAL_EDIT */
 extern int fast_reload_w;
 extern int clear_before_exec;
@@ -97,7 +98,7 @@ extern char search_buffer [256];
 extern char cmd_buf [512];
 
 #if HAVE_GNOME
-#define MENU_PANEL current_panel
+#define MENU_PANEL get_current_panel ()
 #define SELECTED_IS_PANEL 1
 #else
 /* The menu panels */
@@ -121,7 +122,7 @@ typedef struct {
     key_callback fn;
 } key_map;
 
-void update_panels (int force_update, char *current_file, char *other_file);
+void update_panels (int force_update, char *current_file);
 void create_panels (void);
 void repaint_screen (void);
 void outrefresh_screen (void);
@@ -137,28 +138,23 @@ extern const int status_using_ncurses;
 void clr_scr (void);
 void restore_console (void);
 
-/* int do_panel_cd (WPanel *panel, char *new_dir); */
-/* FIXME: this declaration is *needed*, but forces every file that
-   includes main.h to include panel.h first!  Really, main.c shouldn't
-   be a repository for all the little auxiliary functions that are used
-   elsewhere.  do_panel_cd should be moved to util.c or somewhere. */
-
 enum cd_enum {
     cd_parse_command,
     cd_exact
 };
 
-int do_cd (char *new_dir, enum cd_enum cd_type);	/* For find.c */
-void change_panel (void);
-void init_sigchld (void);	/* For subshell.c */
-int load_prompt (int fd, void *unused);
+int do_cd           (char *new_dir, enum cd_enum cd_type);	/* For find.c */
+void change_panel   (void);
+void init_sigchld   (void);	/* For subshell.c */
+int load_prompt     (int fd, void *unused);
 void save_cwds_stat (void);
 void copy_prog_name (void);
-int quiet_quit_cmd (void);	/* For cmd.c and command.c */
+int quiet_quit_cmd  (void);	/* For cmd.c and command.c */
+int quit_cmd        (void);
 
-void untouch_bar (void);
-void touch_bar (void);
-void load_hint (void);
+void untouch_bar    (void);
+void touch_bar      (void);
+void load_hint      (void);
 
 void print_vfs_message(char *msg, ...);
 
@@ -178,8 +174,21 @@ extern WLabel      *process_status;
 extern WMenu      *the_menubar;
 
 extern Dlg_head *midnight_dlg;
+
+/* Back hack to define the following routines only if the client code
+ * has included panel.h
+ */
+#ifdef __PANEL_H
+void directory_history_add   (WPanel *panel, char *s);
+int  do_panel_cd             (WPanel *panel, char *new_dir, enum cd_enum cd_type);
+void update_one_panel_widget (WPanel *panel, int force_update, char *current_file);
 #endif
 
+#endif
+
+void edition_pre_exec (void);
+void edition_post_exec (void);
+
 #ifdef OS2_NT
 #    define MC_BASE ""
 #else
