@@ -105,7 +105,7 @@ main (void)
 
 	case seen_m:
 	    if (c == ' '){
-		state = header_new;
+		state = definition;
 		printf ("_\bF_\br_\bo_\bm ");
 	    } else {
 		state = plain;
@@ -115,6 +115,11 @@ main (void)
 		
 	case header_new:
 	    space_seen = 0;
+            if (c == ' ' || c == '\t') {
+                state = definition;
+                putchar (c);
+                break;
+            }
 	    if (c == '\n'){
 		state = plain;
 		putchar (c);
