@@ -59,7 +59,7 @@
 
 #include "x.h"
 
-/* "$Id: screen.c,v 1.26 1998/05/12 04:26:30 unammx Exp $" */
+/* "$Id: screen.c,v 1.30 1998/09/11 23:27:51 timur Exp $" */
 #define ELEMENTS(arr) ( sizeof(arr) / sizeof((arr)[0]) )
 
 /* If true, show the mini-info on the panel */
@@ -209,7 +209,7 @@ file_entry_color (file_entry *fe)
             return (SPECIAL_COLOR);
         else if (is_exe (fe->buf.st_mode))
             return (EXECUTABLE_COLOR);
-	else if (fe->fname && (strcmp (fe->fname, "core") == 0))
+        else if (fe->fname && (!strcmp (fe->fname, "core") || !strcmp (extension(fe->fname), "core")))
 	    return (CORE_COLOR);
     }
     return (NORMAL_COLOR);
@@ -828,12 +828,8 @@ Xtry_to_select (WPanel *panel, char *name)
     }
 
     /* We only want the last component of the directory */
-    for (subdir = name + strlen (name) - 1; subdir >= name; subdir--){
-	if (*subdir == PATH_SEP){
-	    subdir++;
-	    break;
-	}
-    }
+    subdir = x_basename(name);
+    
     if (subdir < name)
 	subdir = name;
     
@@ -1622,7 +1618,7 @@ do_move_up (WPanel *panel)
     select_item (panel);
 }
 
-static int
+static void
 move_rel (WPanel *panel, int rel)
 {
     unselect_item (panel);
@@ -2136,8 +2132,8 @@ static key_map panel_keymap [] = {
     { KEY_UP, 	move_up },
 
     /* The action button :-) */
-    { '\n',       do_enter },
-    { KEY_ENTER,  do_enter },
+    { '\n',       (key_callback) do_enter },
+    { KEY_ENTER,  (key_callback) do_enter },
 
     { KEY_IC,     mark_file },
     { KEY_HOME,	  move_home },
