@@ -63,9 +63,10 @@
 extern int use_netrc;
 extern int ftpfs_retry_seconds;
 extern int ftpfs_use_passive_connections;
+extern int ftpfs_use_unix_list_options;
 #endif
 
-/* "$Id: setup.c,v 1.9 1998/05/12 04:26:32 unammx Exp $" */
+/* "$Id: setup.c,v 1.10 1998/09/15 19:41:23 norbert Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -225,6 +226,7 @@ static struct {
     { "ftpfs_retry_seconds", &ftpfs_retry_seconds },
     { "ftpfs_always_use_proxy", &ftpfs_always_use_proxy },
     { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
+    { "ftpfs_use_unix_list_options", &ftpfs_use_passive_connections },
 #endif
 #endif
 #ifdef USE_INTERNAL_EDIT
