@@ -72,6 +72,10 @@
 #   include <grp.h>	/* For the group struct & getgrnam()  */
 #endif /* SCO_FLAVOR */
 
+#ifdef __QNX__
+#   include <unix.h>	/* exec*() from <process.h> */
+#endif
+
 #include "dir.h"	/* Required by panel.h below	      */
 #include "util.h"	/* Required by panel.h		      */
 #include "panel.h"	/* For WPanel and current_panel	      */
@@ -280,7 +284,7 @@ void init_subshell (void)
 	    if ((subshell_pipe[READ] = open (tcsh_fifo, O_RDWR)) == -1 ||
 		(subshell_pipe[WRITE] = open (tcsh_fifo, O_RDWR)) == -1)
 	    {
-		fprintf (stderr, "Couldn't open named pipe %s\n", tcsh_fifo);
+		fprintf (stderr, _("Couldn't open named pipe %s\n"), tcsh_fifo);
 		perror (__FILE__": open");
 		use_subshell = FALSE;
 		return;
@@ -502,7 +506,7 @@ void init_subshell (void)
 	goto write_it;
 
 	case TCSH:
-	sprintf (precmd, "alias precmd 'echo $cwd>>%s;kill -STOP $$'\n", tcsh_fifo);
+	sprintf (precmd, "alias precmd 'echo $cwd:q >>%s;kill -STOP $$'\n", tcsh_fifo);
 
 	write_it:
 	write (subshell_pty, precmd, strlen (precmd));
@@ -693,8 +697,8 @@ int exit_subshell (void)
     int quit = TRUE;
     
     if (subshell_state != INACTIVE && subshell_alive)
-	quit = !query_dialog (" Warning ", " The shell is still active."
-			      " Quit anyway? ", 0, 2, "&Yes", "&No");
+	quit = !query_dialog (_(" Warning "), _(" The shell is still active. Quit anyway? "),
+			      0, 2, _("&Yes"), _("&No"));
 
 #if AIX_TCSH_CODE_BELOW_IS_IT_FIXED
     /* New Test code */
@@ -719,8 +723,8 @@ int exit_subshell (void)
 	if (read (subshell_pipe[READ], pty_buffer, pty_buffer_size) == 5)
 	    quit = TRUE;
 	else
-	    quit = !query_dialog (" Warning ", " There are stopped jobs."
-				  " Quit anyway? ", 0, 2, "&Yes", "&No");
+	    quit = !query_dialog (_(" Warning "), _(" There are stopped jobs.")
+				  _(" Quit anyway? "), 0, 2, _("&Yes"), _("&No"));
 
 	synchronize ();
 	subshell_state = RUNNING_COMMAND;
@@ -757,8 +761,6 @@ void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
 	return;
     }
     
-    temp = strdup (directory);
-	
     /* The initial space keeps this out of the command history (in bash
        because we set "HISTCONTROL=ignorespace") */
     write (subshell_pty, " cd ", 4);
@@ -766,15 +768,16 @@ void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
 	temp = name_quote (directory, 0);
 	write (subshell_pty, temp, strlen (temp));    
         free (temp);
-    } else
+    } else {
 	write (subshell_pty, "/", 1);
+    }
     write (subshell_pty, "\n", 1);
     
     subshell_state = RUNNING_COMMAND;
     feed_subshell (QUIETLY, FALSE);
     
     if (subshell_alive && strcmp (subshell_cwd, cpanel->cwd) && strcmp (cpanel->cwd, "."))
-	fprintf (stderr, "Warning: Couldn't change to %s.\n", cpanel->cwd);
+	fprintf (stderr, _("Warning: Couldn't change to %s.\n"), cpanel->cwd);
 
     if (reset_prompt)
 	prompt_pos = 0;
@@ -1195,8 +1198,6 @@ static int pty_open_slave (const char *pty_name)
 
 /* }}} */
 
-#else
-const int use_subshell = 0;
 #endif /* HAVE_SUBSHELL_SUPPORT */
 
 /* {{{ Emacs local variables */
