@@ -50,7 +50,8 @@ typedef struct {
     struct hexedit_change_node *change_list;   /* Linked list of changes */
 
     int dirty;			/* Number of skipped updates */
-
+    int wrap_mode;		/* wrap_mode */
+	
     /* Mode variables */
     int hex_mode;		/* Hexadecimal mode flag */
     int bytes_per_line;		/* Number of bytes per line in hex mode */
@@ -87,7 +88,18 @@ typedef struct {
     int  last_col;		/* last column used */
     int  status_shown;		/* Have we show the file information? */
 #endif
-
+	
+#ifdef HAVE_GNOME
+    int  current_x, current_y;	/* Current x,y position */
+    int  color;			/* Current color */
+    void *gtk_fname;		/* filename widget */
+    void *gtk_offset;		/* offset widget */
+    void *gtk_bytes;		/* bytes */
+    void *gtk_flags;		/* flags (growing) */
+    void *gtk_percent;		/* percent */
+    void *sadj;			/* scrollbar adjustment */
+#endif
+	
     int  move_dir;		/* return value from widget:  
 				 * 0 do nothing
 				 * -1 view previous file
@@ -105,15 +117,21 @@ int view_init (WView *view, char *_command, char *_file, int start_line);
 int view_file (char *filename, int normal, int internal);
 
 /* Internal view routines */
-void view_update (WView *view);
-void view_labels (WView *view);
-int view_event (WView *view, Gpm_Event *event,int *result);
-void toggle_wrap_mode (WView *);
-void toggle_hex_mode (WView *);
-void goto_line (WView *);
-void regexp_search_cmd (WView *);
-void normal_search_cmd (WView *);
-
+void view_status        (WView *);
+void view_percent       (WView *, int, int);
+void view_update        (WView *view);
+void view_labels        (WView *view);
+int view_event          (WView *view, Gpm_Event *event,int *result);
+void toggle_wrap_mode   (WView *);
+void toggle_hex_mode    (WView *);
+void goto_line          (WView *);
+void regexp_search_cmd  (WView *);
+void normal_search_cmd  (WView *);
+void continue_search    (WView *);
+void change_nroff       (WView *view);
+void set_monitor        (WView *view, int set_on);
+void view_move_forward  (WView *view, int i);
+void view_move_backward (WView *view, int i);
 #endif
 
 /* Command: view a file, if _command != NULL we use popen on _command */
@@ -123,7 +141,7 @@ int view (char *_command, char *_file, int *move_direction, int start_line);
 
 extern int mouse_move_pages_viewer;
 extern int max_dirt_limit;
-extern int wrap_mode;
+extern int global_wrap_mode;
 extern int have_fast_cpu;
 extern int default_hex_mode;
 extern int default_magic_flag;
@@ -141,4 +159,37 @@ struct hexedit_change_node {
    unsigned char              value;
 };
 
+#ifdef HAVE_TK
+#define DEF_COLOR         0
+#define BOLD_COLOR        1
+#define UNDERLINE_COLOR   2
+#define MARK_COLOR        3
+#endif
+
+#ifdef HAVE_X
+#ifdef WANT_WIDGETS
+void view_add_character (WView *view, int c);
+void view_add_string    (WView *view, char *s);
+void view_gotoyx        (WView *view, int r, int c);
+void view_set_color     (WView *view, int font);
+void view_display_clean (WView *view, int h, int w);
+
+void x_destroy_view     (WView *);
+void x_create_viewer    (WView *);
+void x_focus_view       (WView *);
+void x_init_view        (WView *);
+
+#ifdef PORT_HAS_VIEW_FREEZE
+void view_freeze (WView *view);
+void view_thaw (WView *view);
+#endif
+
+#endif
+#else
+#    define x_init_view(x)
+#    define x_destroy_view(x)
+#    define x_create_viewer(x)
+#    define x_focus_view(x)
+#endif
+
 #endif	/* __VIEW_H */
