@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.17 1998/05/24 22:22:05 unammx Exp $" */
+/* "$Id: widget.c,v 1.19 1998/05/31 20:28:17 norbert Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -828,7 +828,7 @@ update_input (WInput *in, int clear_first)
 #ifndef HAVE_XVIEW
     int has_history = 0;
     int    i, j;
-    char   c;
+    unsigned char   c;
     int    buf_len = strlen (in->buffer);
 
     if (should_show_history_button (in))
@@ -1049,7 +1049,10 @@ char *show_hist (Hist *history, int widget_x, int widget_y)
 		h = min(h, LINES - y);
 	}
 
-	x = widget_x - 2;
+	if (widget_x > 2)
+	    x = widget_x - 2;
+        else
+            x = 0;
 	if ((w = maxlen + 4) + x > COLS)
 	{
 		w = min(w,COLS);
@@ -1536,7 +1539,10 @@ is_in_input_map (WInput *in, int c_code)
 static void
 port_region_marked_for_delete (WInput *in)
 {
+	if (in->first == 1 && (in->point == in->mark))
+		in->point = strlen (in->buffer);
 	kill_region (in);
+	in->first = 0;
 }
 #else
 static void
@@ -1731,7 +1737,6 @@ input_new (int y, int x, int color, int len, char *def_text, char *tkname)
 
     strcpy (in->buffer, def_text);
     in->point = strlen (in->buffer);
-    in->first = 1;
     return in;
 }
 
