@@ -35,7 +35,7 @@
 #include "layout.h"
 #include "global.h"
 
-/* "$Id: win.c,v 1.10 1995/02/21 19:07:47 miguel Exp $" */
+/* "$Id: win.c,v 1.2 1998/03/25 05:17:01 unammx Exp $" */
 
 typedef void (*fnptr)(void);
 
@@ -213,55 +213,55 @@ int is_quit_char (int c)
 key_code_name_t key_name_conv_tab [] = {
 /* KEY_F(0) is not here, since we are mapping it to f10, so there is no reason
    to define f0 as well. Also, it makes Learn keys a bunch of problems :( */
-    { KEY_F(1),      "f1", "Function key 1" },
-    { KEY_F(2),      "f2", "Function key 2" },
-    { KEY_F(3),      "f3", "Function key 3" },
-    { KEY_F(4),      "f4", "Function key 4" },
-    { KEY_F(5),      "f5", "Function key 5" },
-    { KEY_F(6),      "f6", "Function key 6" },
-    { KEY_F(7),      "f7", "Function key 7" },
-    { KEY_F(8),      "f8", "Function key 8" },
-    { KEY_F(9),      "f9", "Function key 9" },
-    { KEY_F(10),     "f10", "Function key 10" },
-    { KEY_F(11),     "f11", "Function key 11" },
-    { KEY_F(12),     "f12", "Function key 12" },
-    { KEY_F(13),     "f13", "Function key 13" },
-    { KEY_F(14),     "f14", "Function key 14" },
-    { KEY_F(15),     "f15", "Function key 15" },
-    { KEY_F(16),     "f16", "Function key 16" },
-    { KEY_F(17),     "f17", "Function key 17" },
-    { KEY_F(18),     "f18", "Function key 18" },
-    { KEY_F(19),     "f19", "Function key 19" },
-    { KEY_F(20),     "f20", "Function key 20" },
-    { KEY_BACKSPACE, "bs", "Backspace key" },
-    { KEY_END,       "end", "End key" },
-    { KEY_UP,        "up", "Up arrow key" },
-    { KEY_DOWN,      "down", "Down arrow key" },
-    { KEY_LEFT,      "left", "Left arrow key" },
-    { KEY_RIGHT,     "right", "Right arrow key" },
-    { KEY_HOME,      "home", "Home key" },
-    { KEY_NPAGE,     "pgdn", "Page Down key" },
-    { KEY_PPAGE,     "pgup", "Page Up key" },
-    { KEY_IC,        "insert", "Insert key" },
-    { KEY_DC,        "delete", "Delete key" },
-    { ALT('\t'),     "complete", "Completion/M-tab" },
-    { KEY_KP_ADD,    "kpplus", "+ on keypad" },
-    { KEY_KP_SUBTRACT,"kpminus", "- on keypad" },
-    { KEY_KP_MULTIPLY,"kpasterix", "* on keypad" },
+    { KEY_F(1),      "f1",         N_("Function key 1") },
+    { KEY_F(2),      "f2",         N_("Function key 2") },
+    { KEY_F(3),      "f3",         N_("Function key 3") },
+    { KEY_F(4),      "f4",         N_("Function key 4") },
+    { KEY_F(5),      "f5",         N_("Function key 5") },
+    { KEY_F(6),      "f6",         N_("Function key 6") },
+    { KEY_F(7),      "f7",         N_("Function key 7") },
+    { KEY_F(8),      "f8",         N_("Function key 8") },
+    { KEY_F(9),      "f9",         N_("Function key 9") },
+    { KEY_F(10),     "f10",        N_("Function key 10") },
+    { KEY_F(11),     "f11",        N_("Function key 11") },
+    { KEY_F(12),     "f12",        N_("Function key 12") },
+    { KEY_F(13),     "f13",        N_("Function key 13") },
+    { KEY_F(14),     "f14",        N_("Function key 14") },
+    { KEY_F(15),     "f15",        N_("Function key 15") },
+    { KEY_F(16),     "f16",        N_("Function key 16") },
+    { KEY_F(17),     "f17",        N_("Function key 17") },
+    { KEY_F(18),     "f18",        N_("Function key 18") },
+    { KEY_F(19),     "f19",        N_("Function key 19") },
+    { KEY_F(20),     "f20",        N_("Function key 20") },
+    { KEY_BACKSPACE, "bs",         N_("Backspace key") },
+    { KEY_END,       "end",        N_("End key") },
+    { KEY_UP,        "up",         N_("Up arrow key") },
+    { KEY_DOWN,      "down",       N_("Down arrow key") },
+    { KEY_LEFT,      "left",       N_("Left arrow key") },
+    { KEY_RIGHT,     "right",      N_("Right arrow key") },
+    { KEY_HOME,      "home",   	   N_("Home key") },
+    { KEY_NPAGE,     "pgdn",   	   N_("Page Down key") },
+    { KEY_PPAGE,     "pgup",   	   N_("Page Up key") },
+    { KEY_IC,        "insert",     N_("Insert key") },
+    { KEY_DC,        "delete",     N_("Delete key") },
+    { ALT('\t'),     "complete",   N_("Completion/M-tab") },
+    { KEY_KP_ADD,    "kpplus",     N_("+ on keypad") },
+    { KEY_KP_SUBTRACT,"kpminus",   N_("- on keypad") },
+    { KEY_KP_MULTIPLY,"kpasterix", N_("* on keypad") },
 /* From here on, these won't be shown in Learn keys (no space) */    
-    { KEY_LEFT,	     "kpleft", "Left arrow keypad" },
-    { KEY_RIGHT,     "kpright", "Right arrow keypad" },
-    { KEY_UP,	     "kpup", "Up arrow keypad" },
-    { KEY_DOWN,	     "kpdown", "Down arrow keypad" },
-    { KEY_HOME,	     "kphome", "Home on keypad" },
-    { KEY_END,	     "kpend", "End on keypad" },
-    { KEY_NPAGE,     "kpnpage", "Page Down keypad" },
-    { KEY_PPAGE,     "kpppage", "Page Up keypad" },
-    { KEY_IC,        "kpinsert","Insert on keypad" },
-    { KEY_DC,        "kpdelete","Delete on keypad" },
-    { (int) '\n',    "kpenter", "Enter on keypad" },
-    { (int) '/',     "kpslash", "Slash on keypad" },
-    { (int) '#',     "kpnumlock", "NumLock on keypad" },
+    { KEY_LEFT,	     "kpleft",     N_("Left arrow keypad") },
+    { KEY_RIGHT,     "kpright",    N_("Right arrow keypad") },
+    { KEY_UP,	     "kpup",       N_("Up arrow keypad") },
+    { KEY_DOWN,	     "kpdown",     N_("Down arrow keypad") },
+    { KEY_HOME,	     "kphome",     N_("Home on keypad") },
+    { KEY_END,	     "kpend",      N_("End on keypad") },
+    { KEY_NPAGE,     "kpnpage",    N_("Page Down keypad") },
+    { KEY_PPAGE,     "kpppage",    N_("Page Up keypad") },
+    { KEY_IC,        "kpinsert",   N_("Insert on keypad") },
+    { KEY_DC,        "kpdelete",   N_("Delete on keypad") },
+    { (int) '\n',    "kpenter",    N_("Enter on keypad") },
+    { (int) '/',     "kpslash",    N_("Slash on keypad") },
+    { (int) '#',     "kpnumlock",  N_("NumLock on keypad") },
     { 0, 0 }
     };
 
