@@ -28,7 +28,7 @@
 
 /* }}} */
 
-/*  [] = "$Id: wtools.c,v 1.3 1995/02/21 19:07:51 miguel Exp $" */
+/*  [] = "$Id: wtools.c,v 1.10 1998/05/19 18:37:08 unammx Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -111,7 +111,7 @@ Listbox *create_listbox_window (int cols, int lines, char *title, char *help)
 {
     int xpos, ypos, len;
     Listbox  *listbox = xmalloc (sizeof (Listbox), "create_listbox_window");
-    char* cancel_string = "&Cancel";
+    char* cancel_string = _("&Cancel");
 
     /* Adjust sizes */
     lines = (lines > LINES-6) ? LINES - 6 : lines;
@@ -221,15 +221,23 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
     query_colors [2] = (flags & D_ERROR) ? ERROR_COLOR : COLOR_HOT_NORMAL;
     query_colors [3] = (flags & D_ERROR) ? COLOR_HOT_NORMAL :  COLOR_HOT_FOCUS;
     
+    if (header == MSG_ERROR)
+	    header = _(" Error ");
+    
     if (count > 0){
 	va_start (ap, count);
 	for (i = 0; i < count; i++)
-	    win_len += strlen (va_arg (ap, char *)) + 6;
+	{
+		char* cp = va_arg (ap, char *);
+	    win_len += strlen (cp) + 6;
+		if (strchr (cp, '&') != NULL)
+			win_len--;
+	}
 	va_end (ap);
     }
 
     /* count coordinates */
-    cols = 6 + max (win_len+1, max (strlen (header), msglen (text, &lines)));
+    cols = 6 + max (win_len, max (strlen (header), msglen (text, &lines)));
     lines += 4 + (count > 0 ? 2 : 0);
     xpos = COLS/2 - cols/2;
     ypos = LINES/3 - (lines-3)/2;
@@ -255,6 +263,8 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	for (i = 0; i < count; i++){
 	    cur_name = va_arg (ap, char *);
 	    xpos = strlen (cur_name)+6;
+		if (strchr(cur_name, '&') != NULL)
+			xpos--;
 #ifndef HAVE_XVIEW
 	    add_widget (query_dlg, button_new
 			(lines-3, cols, B_USER+i, NORMAL_BUTTON, cur_name,
@@ -289,7 +299,7 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	destroy_dlg (query_dlg);
     } else {
 #ifdef HAVE_X
-	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, NORMAL_BUTTON, "&Ok", 0, 0, NULL),
+	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, NORMAL_BUTTON, _("&Ok"), 0, 0, NULL),
 	    XV_WLAY_CENTERROW);
 
 	add_widget (query_dlg, label_new (2, 3, text, NULL));
@@ -392,10 +402,10 @@ Chooser *new_chooser (int lines, int cols, char *help, int flags)
     
     if (button_lines){
 	add_widget (c->dialog, button_new (lines-button_lines+1,
-				20, B_ENTER, DEFPUSH_BUTTON, "&Remove",
+				20, B_ENTER, DEFPUSH_BUTTON, _("&Remove"),
 				remove_callback, c, "button-remove"));
 	add_widget (c->dialog, button_new (lines-button_lines+1,
-				    4, B_CANCEL, NORMAL_BUTTON, "&Cancel",
+				    4, B_CANCEL, NORMAL_BUTTON, _("&Cancel"),
 				    0, 0, "button-cancel"));
     }		    
     add_widget (c->dialog, c->listbox);
@@ -430,18 +440,19 @@ static int quick_callback (struct Dlg_head *h, int id, int Msg)
 	dlg_move (h, 1,((h->cols-strlen (h->data))/2));
 	addstr (h->data);
 	break;
-
+#endif	
     case DLG_KEY:
 	if (id == '\n'){
-	    h->running = 0;
 	    h->ret_value = B_ENTER;
+	    dlg_stop (h);
 	    break;
 	}
-#endif	
     }
     return 0;
 }
 
+#define I18N(x) (do_int && *x ? (x = _(x)): x)
+
 int quick_dialog_skip (QuickDialog *qd, int nskip)
 {
     Dlg_head *dd;
@@ -452,6 +463,15 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
     int      return_val;
     WInput   *input;
     QuickWidget *qw;
+    int      do_int;
+
+    if (!qd->i18n){
+	qd->i18n = 1;
+	do_int = 1;
+	if (*qd->title)
+	    qd->title = _(qd->title);
+    } else
+	do_int = 0;
     
     if (qd->xpos == -1)
         dd = create_dlg (0, 0, qd->ylen, qd->xlen, dialog_colors, quick_callback,
@@ -469,14 +489,16 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
     dd->data  = qd->title;
 
     for (qw = qd->widgets; qw->widget_type; qw++){
-#ifndef HAVE_X
+#ifdef HAVE_X
+	xpos = ypos = 0;
+#else
 	xpos = (qd->xlen * qw->relative_x)/qw->x_divisions;
 	ypos = (qd->ylen * qw->relative_y)/qw->y_divisions;
 #endif
 	
 	switch (qw->widget_type){
 	case quick_checkbox:
-	    widget = check_new (ypos, xpos, *qw->result, qw->text, qw->tkname);
+	    widget = check_new (ypos, xpos, *qw->result, I18N (qw->text), qw->tkname);
 	    break;
 
 	case quick_radio:
@@ -487,7 +509,7 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
 	    
 	case quick_button:
 	    widget = button_new (ypos, xpos, qw->value, (qw->value==B_ENTER) ? DEFPUSH_BUTTON : NORMAL_BUTTON,
-	    qw->text, 0, 0, qw->tkname);
+	    I18N (qw->text), 0, 0, qw->tkname);
 	    break;
 
 	    /* We use the hotkey pos as the field length */
@@ -502,7 +524,7 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
 	    break;
 
 	case quick_label:
-	    widget = label_new (ypos, xpos, qw->text, qw->tkname);
+	    widget = label_new (ypos, xpos, I18N(qw->text), qw->tkname);
 	    break;
 	    
 	default:
@@ -556,9 +578,9 @@ char *real_input_dialog_help (char *header, char *text, char *help, char *def_te
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 6, 10, 1, 0, "&Cancel", 0, B_CANCEL, 0, 0,
+    { quick_button, 6, 10, 1, 0, N_("&Cancel"), 0, B_CANCEL, 0, 0,
 	  XV_WLAY_RIGHTOF, "button-cancel" },
-    { quick_button, 3, 10, 1, 0, "&Ok", 0, B_ENTER, 0, 0,
+    { quick_button, 3, 10, 1, 0, N_("&Ok"), 0, B_ENTER, 0, 0,
 	  XV_WLAY_CENTERROW, "button-ok" },
     { quick_input,  4, 80, 0, 0, "", 58, 0, 0, 0, XV_WLAY_NEXTROW, 0 },
     { quick_label,  3, 80, 2, 0, "", 0, 0, 0, 0, XV_WLAY_NEXTROW, "label" },
@@ -584,18 +606,30 @@ char *real_input_dialog_help (char *header, char *text, char *help, char *def_te
     len = max (strlen (header), msglen (text, &lines)) + 4;
     len = max (len, 64);
 
-    if (strncmp (text, "Password", 8) == 0){
+    if (strncmp (text, _("Password:"), strlen (_("Password"))) == 0){
 	quick_widgets [INPUT_INDEX].value = 1;
 	tk_name[3]=0;
     } else {
 	quick_widgets [INPUT_INDEX].value = 0;
     }
+
+#ifdef ENABLE_NLS
+	/* 
+	 * An attempt to place buttons symmetrically, based on actual i18n
+	 * length of the string. It looks nicer with i18n (IMO) - alex
+	 */
+	quick_widgets [0].relative_x = len/2 + 4;
+	quick_widgets [1].relative_x = 
+		len/2 - (strlen (_(quick_widgets [1].text)) + 9);
+	quick_widgets [0].x_divisions = quick_widgets [1].x_divisions = len;
+#endif /* ENABLE_NLS */
     
     Quick_input.xlen  = len;
     Quick_input.xpos  = -1;
     Quick_input.title = header;
     Quick_input.help  = help;
     Quick_input.class = "quick_input";
+    Quick_input.i18n  = 0;
     quick_widgets [INPUT_INDEX+1].text = text;
     quick_widgets [INPUT_INDEX].text = def_text;
 
@@ -624,9 +658,10 @@ int input_dialog_help_2 (char *header, char *text1, char *text2, char *help, cha
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 6, 10, 4, 0, "Cancel", 0, B_CANCEL, 0, 0,
+    { quick_button, 6, 10, 4, 0, N_("&Cancel"), 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 3, 10, 4, 0, "Ok", 0, B_ENTER, 0, 0,
+    { quick_button, 3, 10, 4, 0, N_("Ok")
+, 0, B_ENTER, 0, 0,
 	  XV_WLAY_DONTCARE, "button-ok" },
     { quick_input,  4, 80, 4, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
     { quick_label,  3, 80, 3, 0, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
@@ -648,6 +683,7 @@ int input_dialog_help_2 (char *header, char *text1, char *text2, char *help, cha
     Quick_input.title = header;
     Quick_input.help  = help;
     Quick_input.class = "quick_input_2";
+    Quick_input.i18n  = 0;
     quick_widgets [5].text = text1;
     quick_widgets [3].text = text2;
 
