@@ -2,11 +2,11 @@ set props(.quick_input.columns) 3
 set props(.quick_input.rows)    5
 set props(.quick_input.frames)  1
 set props(.quick_input.texts)   0
-set props(.quick_input.components)   "button-ok button-cancel label input "
+set props(.quick_input.components)   "button-ok button-cancel label inp "
 set props(.quick_input.count)   4
 set props(.quick_input.props.button-ok) "-in .quick_input.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
 set props(.quick_input.props.button-cancel) "-in .quick_input.widgets -column 5 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
 set props(.quick_input.props.label) "-in .quick_input.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
-set props(.quick_input.props.input) "-in .quick_input.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
+set props(.quick_input.props.inp) "-in .quick_input.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
 set props(.quick_input.frame0) "-in .quick_input.widgets -column 1 -row 1 -columnspan 7 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
 set props(.quick_input.relief.frame0) groove
