@@ -3,31 +3,39 @@
 #if !defined(__FTPFS_H)
 #define __FTPFS_H
 
-struct ftpentry
+struct direntry
 {
     char *name;
     int count;
     char *linkname;
-    char *local_filename;
+    char *local_filename, *remote_filename;
     int local_is_temp:1;
     int freshly_created:1;
+    int linear:1;		/* Did they promiss not to seek? */
     int tmp_reget;
     struct stat local_stat;
-    char *remote_filename;
     struct stat s;
     struct stat *l_stat;
-    struct ftpfs_connection *bucket;
+    struct connection *bucket;
+
+    int data_sock;		/* For linear_ operations */
 };
 
-struct ftpfs_dir
+struct dir
 {
     int count;
     struct timeval timestamp;
     char *remote_path;
     struct linklist *file_list;
+    int symlink_status;
 };
 
-struct ftpfs_connection {
+/* valid values for dir->symlink_status */
+#define FTPFS_NO_SYMLINKS          0
+#define FTPFS_UNRESOLVED_SYMLINKS  1
+#define FTPFS_RESOLVED_SYMLINKS    2
+
+struct connection {
     char *host;
     char *user;
     char *current_directory;
@@ -47,6 +55,9 @@ struct ftpfs_connection {
     int isbinary;
     int cwd_defered;  /* current_directory was changed but CWD command hasn't
                          been sent yet */
+    int strict_rfc959_list_cmd; /* ftp server doesn't understand 
+                                   "LIST -la <path>"; use "CWD <path>"/
+                                   "LIST" instead */
 };
 
 #define qhost(b) (b)->host
@@ -63,9 +74,9 @@ struct ftpfs_connection {
 /* Increased since now we may use C-r to reread the contents */
 #define FTPFS_DIRECTORY_TIMEOUT 30 * 60
 
-#define FTPFS_RESOLVE_SYMLINK 1
-#define FTPFS_OPEN            2
-#define FTPFS_FREE_RESOURCE   4
+#define DO_RESOLVE_SYMLINK 1
+#define DO_OPEN            2
+#define DO_FREE_RESOURCE   4
 
 extern char *ftpfs_anonymous_passwd;
 extern char *ftpfs_proxy_host;
