@@ -13,13 +13,17 @@
 
 #include "vfs.h"
 
+/* Note: Some of this functions are not static. This has rather good
+ * reason: exactly same functions would have to appear in sfs.c. This
+ * saves both computer's memory and my work.  <pavel@ucw.cz>
+ * */
     
 static void *local_open (char *file, int flags, int mode)
 {
     int *local_info;
     int fd;
 
-    fd = open (file, flags, mode);
+    fd = open (file, NO_LINEAR(flags), mode);
     if (fd == -1)
 	return 0;
 
@@ -29,7 +33,7 @@ static void *local_open (char *file, int flags, int mode)
     return local_info;
 }
 
-static int local_read (void *data, char *buffer, int count)
+int local_read (void *data, char *buffer, int count)
 {
     int n;
 
@@ -48,7 +52,7 @@ static int local_read (void *data, char *buffer, int count)
     return n;
 }
 
-static int local_close (void *data)
+int local_close (void *data)
 {
     int fd;
 
@@ -60,7 +64,7 @@ static int local_close (void *data)
     return close (fd);
 }
 
-static int local_errno (void)
+int local_errno (void)
 {
     return errno;
 }
@@ -80,6 +84,16 @@ static void *local_opendir (char *dirname)
     return local_info;
 }
 
+static int local_telldir (void *data)
+{
+    return telldir( *(DIR **) data );
+}
+
+static void local_seekdir (void *data, int offset)
+{
+    seekdir( *(DIR **) data, offset );
+}
+
 static void *local_readdir (void *data)
 {
     return readdir (*(DIR **) data);
@@ -109,7 +123,7 @@ static int local_lstat (char *path, struct stat *buf)
 #endif
 }
 
-static int local_fstat (void *data, struct stat *buf)
+int local_fstat (void *data, struct stat *buf)
 {
     return fstat (*((int *) data), buf);    
 }
@@ -175,7 +189,7 @@ static int local_chdir (char *path)
     return chdir (path);
 }
 
-static int local_lseek (void *data, off_t offset, int whence)
+int local_lseek (void *data, off_t offset, int whence)
 {
     int fd = * (int *) data;
 
@@ -249,6 +263,8 @@ vfs local_vfs_ops = {
     local_opendir,
     local_readdir,
     local_closedir,
+    local_telldir,
+    local_seekdir,
 
     local_stat,
     local_lstat,
