@@ -2,6 +2,7 @@
    Copyright (C) 1995 The Free Software Foundation
    
    Written by: 1995 Jakub Jelinek
+   Rewritten by: 1998 Pavel Machek
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,8 +38,10 @@
 #include <time.h>
 #include "../src/fs.h"
 #include "../src/util.h"
+#include "../src/dialog.h"	/* For MSG_ERROR */
 #include "../src/mem.h"
 #include "../src/mad.h"
+#include "../src/main.h"
 #include "vfs.h"
 #include "tar.h"
 #include "names.h"
@@ -76,19 +79,19 @@ long from_oct (int digs, char *where)
     return value;
 }
 
-static struct tarfs_archive *first_archive = NULL;
-static int tarerrno = 0;
+static struct archive *first_archive = NULL;
+static int my_errno = 0;
 static struct stat hstat;		/* Stat struct corresponding */
 static char *current_file_name, *current_link_name;
-static struct tarfs_entry *tarfs_find_entry (struct tarfs_entry *dir, char *name, int make_dirs);
+static struct entry *find_entry (struct entry *dir, char *name, int make_dirs, int make_file);
 
 void tarfs_fill_names (void (*func)(char *))
 {
-    struct tarfs_archive *a = first_archive;
+    struct archive *a = first_archive;
     char *name;
     
     while (a){
-	name = copy_strings ("tar:", a->name, "/",
+	name = copy_strings ( a->name, "#tar/",
 			     a->current_dir->name, 0);
 	(*func)(name);
 	free (name);
@@ -96,12 +99,12 @@ void tarfs_fill_names (void (*func)(char *))
     }
 }
 
-static void make_dot_doubledot (struct tarfs_entry *ent)
+static void make_dot_doubledot (struct entry *ent)
 {
-    struct tarfs_entry *entry = (struct tarfs_entry *)
-	xmalloc (sizeof (struct tarfs_entry), "Tar: tarfs_entry");
-    struct tarfs_entry *parentry = ent->dir;
-    struct tarfs_inode *inode = ent->inode, *parent;
+    struct entry *entry = (struct entry *)
+	xmalloc (sizeof (struct entry), "Tar: entry");
+    struct entry *parentry = ent->dir;
+    struct inode *inode = ent->inode, *parent;
     
     parent = (parentry != NULL) ? parentry->inode : NULL;
     entry->name = strdup (".");
@@ -115,8 +118,8 @@ static void make_dot_doubledot (struct tarfs_entry *ent)
     inode->first_in_subdir = entry;
     inode->last_in_subdir = entry;
     inode->nlink++;
-    entry->next_in_dir = (struct tarfs_entry *)
-	xmalloc (sizeof (struct tarfs_entry), "Tar: tarfs_entry");
+    entry->next_in_dir = (struct entry *)
+	xmalloc (sizeof (struct entry), "Tar: entry");
     entry=entry->next_in_dir;
     entry->name = strdup ("..");
     entry->has_changed = 0;
@@ -137,16 +140,16 @@ static void make_dot_doubledot (struct tarfs_entry *ent)
     }
 }
 
-static struct tarfs_entry *generate_entry (struct tarfs_archive *archive, 
-    char *name, struct tarfs_entry *parentry, mode_t mode)
+static struct entry *generate_entry (struct archive *archive, 
+    char *name, struct entry *parentry, mode_t mode)
 {
     mode_t myumask;
-    struct tarfs_inode *inode, *parent; 
-    struct tarfs_entry *entry;
+    struct inode *inode, *parent; 
+    struct entry *entry;
 
     parent = (parentry != NULL) ? parentry->inode : NULL;
-    entry = (struct tarfs_entry *)
-	xmalloc (sizeof (struct tarfs_entry), "Tar: tarfs_entry");
+    entry = (struct entry *)
+	xmalloc (sizeof (struct entry), "Tar: entry");
     
     entry->name = strdup (name);
     entry->has_changed = 0;
@@ -160,8 +163,8 @@ static struct tarfs_entry *generate_entry (struct tarfs_archive *archive,
     	parent->last_in_subdir->next_in_dir = entry;
     	parent->last_in_subdir = entry;
     }
-    inode = (struct tarfs_inode *)
-	xmalloc (sizeof (struct tarfs_inode), "Tar: tarfs_inode");
+    inode = (struct inode *)
+	xmalloc (sizeof (struct inode), "Tar: inode");
     entry->inode = inode;
     inode->local_filename = NULL;
     inode->has_changed = 0;
@@ -201,12 +204,12 @@ static struct tarfs_entry *generate_entry (struct tarfs_archive *archive,
     return entry;
 }
 
-static void free_entries (struct tarfs_entry *entry)
+static void free_entries (struct entry *entry)
 {
     return;
 }
 
-static void free_archive (struct tarfs_archive *archive)
+static void free_archive (struct archive *archive)
 {
     long l;
 
@@ -218,10 +221,8 @@ static void free_archive (struct tarfs_archive *archive)
     	    free (archive->block_ptr);
     	}
     } else {
-        if (archive->is_gzipped == tar_uncompressed_local) {
-    	    mc_unlink (archive->tmpname);
+        if (archive->tmpname)
     	    free (archive->tmpname);
-    	}
 	if (archive->fd != -1)
 	    mc_close(archive->fd);
     }
@@ -244,7 +245,7 @@ static INLINE int gzip_limit_ok (int size)
  *
  * Returns: 0 on failure
  */
-static INLINE int load_compressed_tar (struct tarfs_archive *current_archive,
+static INLINE int load_compressed_tar (struct archive *current_archive,
 					int size, int fd, int type)
 {	
     int pipehandle, i;
@@ -335,76 +336,40 @@ static INLINE int load_compressed_tar (struct tarfs_archive *current_archive,
 }
 
 /* Returns a file handle of the opened local tar file or -1 on error */
-static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
+static INLINE int uncompress_tar_file  (struct archive *current_archive,
 					int size, int fd, int type)
 {
-    FILE *f;
-    char *command;
-    int i, result;
-    int dash_number = 0;
-    char buffer [8192];	/* Changed to 8K: better transfer size */
-    
-    current_archive->is_gzipped = tar_uncompressed_local;
-    current_archive->tmpname = strdup (tmpnam (NULL));
-    
-    /* Some security is sometimes neccessary :) */        
-    command = copy_strings ("touch ", current_archive->tmpname,
-			    " ; chmod 0600 ", current_archive->tmpname, " ; ",
-			    decompress_command (type),
-			    "2>/dev/null >", current_archive->tmpname, NULL);
-    
-    if ((f = popen (command, "w")) == NULL) {
-	mc_close (fd);
-	free_archive (current_archive);
-	free (command);
-	return -1;
-    }
-    free (command);
-    
-    while ((i = mc_read (fd, buffer, sizeof (buffer))) > 0){
-	if ((dash_number++ % 64) == 0)
-	    rotate_dash ();
-	fwrite (buffer, 1, i, f);
-	if (ferror (f)) {
-            pclose (f);
-#ifdef SCO_FLAVOR
-            waitpid(-1,NULL,WNOHANG);
-#endif /* SCO_FLAVOR */
-            mc_close (fd);
-            free_archive (current_archive);
-            return -1;
-        }
-    }
+    int result;
 
-    pclose (f);
-#ifdef SCO_FLAVOR
-    waitpid(-1,NULL,WNOHANG);
-#endif /* SCO_FLAVOR */
-    mc_close (fd);
+  /* From time we have sfs this function is pretty trivial */
+    current_archive->is_gzipped = tar_uncompressed_local;
+    current_archive->tmpname = copy_strings( current_archive->name, 
+					     decompress_extension (type), NULL );
+    
     result = mc_open (current_archive->tmpname, O_RDONLY);
-    if (result == -1){
+    if (result == -1)
 	free_archive (current_archive);
-	return -1;
-    }
     return result;
 }
 
 /* Returns fd of the open tar file */
-static int open_tar_archive (char *name, struct tarfs_archive **pparc)
+static int open_tar_archive (char *name, struct archive **pparc)
 {
     static dev_t __tar_no = 0;
     int result, type;
     long size;
     mode_t mode;
-    struct tarfs_archive *current_archive;
-    static struct tarfs_entry *root_entry;
+    struct archive *current_archive;
+    static struct entry *root_entry;
     
     result = mc_open (name, O_RDONLY);
-    if (result == -1)
+    if (result == -1) {
+        message_2s (1, MSG_ERROR, _("Couldn't open tar archive\n%s"), name);
     	return -1;
+    }
     
-    current_archive = (struct tarfs_archive *)
-                      xmalloc (sizeof (struct tarfs_archive), "Tar archive");
+    current_archive = (struct archive *)
+                      xmalloc (sizeof (struct archive), "Tar archive");
     current_archive->current_tar_position = 0;
     current_archive->name = strdup (name);
     current_archive->__inode_counter = 0;
@@ -413,18 +378,23 @@ static int open_tar_archive (char *name, struct tarfs_archive **pparc)
     current_archive->next = first_archive;
     current_archive->fd_usage = 0;
     current_archive->fd = -1;
+    current_archive->tmpname = NULL;
     size = is_gunzipable (result, &type);
     mc_lseek (result, 0, SEEK_SET);
 
     /* Find out the method to handle this tar file */
     if (size > 0 && gzip_limit_ok (size)) {
-	if (load_compressed_tar (current_archive, size, result, type) == 0)
+        if (load_compressed_tar (current_archive, size, result, type) == 0) {
+	    message_2s (1, MSG_ERROR, _("Couldn't load compressed tar\n%s"), name);
 	    return -1;
+	}
 	result = 0;
     } else if (size > 0) {
 	result = uncompress_tar_file (current_archive, size, result, type);
-	if (result == -1)
+	if (result == -1) {
+	    message_2s (1, MSG_ERROR, _("Couldn't uncompress tar archive\n%s"), name);
 	    return -1;
+	}
     } else {
     	current_archive->is_gzipped = tar_normal;
     }
@@ -453,12 +423,12 @@ static int open_tar_archive (char *name, struct tarfs_archive **pparc)
     return result;
 }
 
-static int get_current_position (struct tarfs_archive *archive, int tard)
+static int get_current_position (struct archive *archive, int tard)
 {
     return archive->current_tar_position;
 }
 
-static union record *find_current_record (struct tarfs_archive *archive, long pos)
+static union record *find_current_record (struct archive *archive, long pos)
 {
     long l, l2;
     static union record *ur;
@@ -475,7 +445,7 @@ static union record *find_current_record (struct tarfs_archive *archive, long po
 
 static union record rec_buf;
 
-static union record *get_next_record (struct tarfs_archive *archive, int tard)
+static union record *get_next_record (struct archive *archive, int tard)
 {
     if (archive->is_gzipped == targz_growing) {
     	bcopy (archive->current_record, rec_buf.charptr, RECORDSIZE);
@@ -486,7 +456,7 @@ static union record *get_next_record (struct tarfs_archive *archive, int tard)
     return &rec_buf;
 }
 
-static void skip_n_records (struct tarfs_archive *archive, int tard, int n)
+static void skip_n_records (struct archive *archive, int tard, int n)
 {
     if (archive->is_gzipped == targz_growing)
     	archive->current_record = find_current_record (archive, archive->current_tar_position + n * RECORDSIZE);
@@ -500,7 +470,7 @@ static void skip_n_records (struct tarfs_archive *archive, int tard, int n)
  * 2 for a record full of zeros (EOF marker).
  *
  */
-static int read_header (struct tarfs_archive *archive, int tard)
+static int read_header (struct archive *archive, int tard)
 {
     register int i;
     register long sum, signed_sum, recsum;
@@ -541,17 +511,25 @@ static int read_header (struct tarfs_archive *archive, int tard)
 
     if (sum == 8 * ' ') {
 	/*
-		 * This is a zeroed record...whole record is 0's except
-		 * for the 8 blanks we faked for the checksum field.
-		 */
+	 * This is a zeroed record...whole record is 0's except
+	 * for the 8 blanks we faked for the checksum field.
+	 */
 	return 2;
     }
     if (sum != recsum && signed_sum != recsum)
 	return 0;
-
+    
     /*
-	 * Good record.  Decode file size and return.
-	 */
+     * linkflag on BSDI tar (pax) always '\000'
+     */
+    if(header->header.linkflag == '\000' &&
+       strlen(header->header.arch_name) &&
+       header->header.arch_name[strlen(header->header.arch_name) - 1] == '/')
+	header->header.linkflag = LF_DIR;
+    
+     /*
+     * Good record.  Decode file size and return.
+     */
     if (header->header.linkflag == LF_LINK || header->header.linkflag == LF_DIR)
 	hstat.st_size = 0;	/* Links 0 size on tape */
     else
@@ -573,7 +551,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	     size -= written) {
 	    data = get_next_record (archive, tard)->charptr;
 	    if (data == NULL) {
-		message_1s (1, " Error ", "Unexpected EOF on archive file");
+		message_1s (1, MSG_ERROR, _("Unexpected EOF on archive file"));
 		return 0;
 	    }
 	    written = RECORDSIZE;
@@ -589,8 +567,8 @@ static int read_header (struct tarfs_archive *archive, int tard)
 #endif
 	goto recurse;
     } else {
-	struct tarfs_entry *entry, *pent;
-	struct tarfs_inode *inode;
+	struct entry *entry, *pent;
+	struct inode *inode;
 	long data_position;
 	char *p, *q;
 	int len;
@@ -625,12 +603,12 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	    q = current_file_name;
 	}
 	    
-	pent = tarfs_find_entry (archive->root_entry, q, 1);
+	pent = find_entry (archive->root_entry, q, 1, 0);
 	if (pent == NULL) {
-	    message_1s (1, " Error ", "Inconsistent tar archive");
+	    message_1s (1, MSG_ERROR, _("Inconsistent tar archive"));
 	}
 
-	entry = (struct tarfs_entry *) xmalloc (sizeof (struct tarfs_entry), "Tar: tarfs_entry");
+	entry = (struct entry *) xmalloc (sizeof (struct entry), "Tar: entry");
 	entry->name = strdup (p);
 	entry->has_changed = 0;
 	entry->header_offset = header_position;
@@ -646,9 +624,9 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	free (current_file_name);
 
 	if (header->header.linkflag == LF_LINK) {
-	    pent = tarfs_find_entry (archive->root_entry, current_link_name, 0);
+	    pent = find_entry (archive->root_entry, current_link_name, 0, 0);
 	    if (pent == NULL) {
-	        message_1s (1, " Error ", "Inconsistent tar archive");
+	        message_1s (1, MSG_ERROR, _("Inconsistent tar archive"));
 	    } else {
 		entry->inode = pent->inode;
 		pent->inode->nlink++;
@@ -662,7 +640,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	        return 1;
 	    }
 	}
-	inode = (struct tarfs_inode *) xmalloc (sizeof (struct tarfs_inode), "Tar: tarfs_inode");
+	inode = (struct inode *) xmalloc (sizeof (struct inode), "Tar: inode");
 	entry->inode = inode;
 	inode->local_filename = NULL;
 	inode->has_changed = 0;
@@ -740,15 +718,15 @@ static int read_header (struct tarfs_archive *archive, int tard)
  * Main loop for reading an archive.
  * Returns 0 on success, -1 on error.
  */
-int read_tar_archive (char *name, struct tarfs_archive **pparc)
+int read_tar_archive (char *name, struct archive **pparc)
 {
     int status = 3;		/* Initial status at start of archive */
     int prev_status;
     int tard;
-    struct tarfs_archive *archive;
+    struct archive *archive;
 
     if ((tard = open_tar_archive (name, &archive)) == -1) {	/* Open for reading */
-        message_2s (1, " Error ", "Couldn't open tar archive\n%s", name);
+        message_2s (1, MSG_ERROR, _("Couldn't open tar archive\n%s"), name);
 	return -1;
     }
 
@@ -767,7 +745,7 @@ int read_tar_archive (char *name, struct tarfs_archive **pparc)
 	case 0:		/* Invalid header */
 	    switch (prev_status) {
 	    case 3:		/* Error on first record */
-		message_2s (1, " Error ", "Hmm,...\n%s\ndoesn't look like a tar archive.", name);
+		message_2s (1, MSG_ERROR, _("Hmm,...\n%s\ndoesn't look like a tar archive."), name);
 		/* FALL THRU */
 	    case 2:		/* Error after record of zeroes */
 	    case 1:		/* Error after header rec */
@@ -791,64 +769,35 @@ int read_tar_archive (char *name, struct tarfs_archive **pparc)
     return 0;
 }
 
-char *tarfs_analysis (char *name, char **archive, int is_dir)
-{
-    static struct {
-        int len; /* strlen (ext)  */
-        char *ext;
-    } tarext[] = {{4, ".tar"},
-                  {4, ".tgz"},
-                  {7, ".tar.gz"},
-                  {4, ".taz"},
-                  {4, ".tpz"},
-		  {6, ".tar.z"},
-		  {7, ".tar.bz"},
-		  {8, ".tar.bz2"},
-                  {6, ".tar.Z"} };
-    char *p, *local;
-    unsigned int i;
-    char *archive_name = NULL;
+static char *get_path (char *inname, struct archive **archive, int is_dir,
+    int do_not_open);
 
-                                               /*  |  this is len of "tar:" plus some minimum
-                                                *  v  space needed for the extension */
-    for (p = name + strlen (name); p >= name + 8; p--)
-    	if (*p == '/' || (is_dir && !*p))
-    	    for (i = 0; i < sizeof (tarext) / sizeof (tarext [0]); i++)
-    	        if (!strncmp (p - tarext [i].len, tarext [i].ext, tarext [i].len)) {
-    	            char c = *p;
-    	            
-    	            *p = 0;
-    	            archive_name = vfs_canon (name + 4);
-    	            *archive = archive_name;
-    	            *p = c;
-    	            local = strdup (p);
-    	            return local;
-    	        }
-    tarerrno = ENOENT;
-    return NULL;
-}
-
-/* Returns allocated path inside the archive or NULL */
-static char *tarfs_get_path (char *inname, struct tarfs_archive **archive, int is_dir,
+/* Returns path inside argument. Returned char* is inside inname, which is mangled	
+ * by this operation (so you must not free it's return value)
+ */
+static char *get_path_mangle (char *inname, struct archive **archive, int is_dir,
     int do_not_open)
 {
     char *local, *archive_name;
     int result = -1;
-    struct tarfs_archive *parc;
+    struct archive *parc;
     struct vfs_stamping *parent;
     vfs *v;
     struct stat stat_buf;
     
-    local = tarfs_analysis (inname, &archive_name, is_dir);
-    if (local == NULL) {
-    	tarerrno = ENOENT;
-    	return NULL;
-    }
+    archive_name = inname;
+    vfs_split( inname, &local, NULL );
+    if (!local)
+        local = "";
 
     mc_stat (archive_name, &stat_buf);
 
     for (parc = first_archive; parc != NULL; parc = parc->next)
         if (!strcmp (parc->name, archive_name)) {
+	    if (vfs_uid && (!(stat_buf.st_mode & 0004)))
+		if ((stat_buf.st_gid != vfs_gid) || !(stat_buf.st_mode & 0040))
+		    if ((stat_buf.st_uid != vfs_uid) || !(stat_buf.st_mode & 0400))
+			return NULL;
 /* Has the cached archive been changed on the disk? */
 	    if (parc->tarstat.st_mtime < stat_buf.st_mtime) { /* Yes, reload! */
 		(*tarfs_vfs_ops.free) ((vfsid) parc);
@@ -863,12 +812,7 @@ static char *tarfs_get_path (char *inname, struct tarfs_archive **archive, int i
         result = -1;
     else
         result = read_tar_archive (archive_name, &parc);
-    if (result == -1) {
-	tarerrno = EIO;
-	free(local);
-	free(archive_name);
-	return NULL;
-    }
+    if (result == -1) ERRNOR (EIO, NULL);
     v = vfs_type (archive_name);
     if (v == &local_vfs_ops) {
 	parent = NULL;
@@ -882,27 +826,26 @@ static char *tarfs_get_path (char *inname, struct tarfs_archive **archive, int i
     vfs_rm_parents (parent);
 return_success:
     *archive = parc;
-    free (archive_name);
     return local;
 }
 
-struct tarfs_loop_protect {
-    struct tarfs_entry *entry;
-    struct tarfs_loop_protect *next;
+struct loop_protect {
+    struct entry *entry;
+    struct loop_protect *next;
 };
 static int errloop;
 static int notadir;
 
-static struct tarfs_entry *
-__tarfs_find_entry (struct tarfs_entry *dir, char *name,
-		    struct tarfs_loop_protect *list, int make_dirs);
+static struct entry *
+__find_entry (struct entry *dir, char *name,
+		    struct loop_protect *list, int make_dirs, int make_file);
 
-static struct tarfs_entry *
-__tarfs_resolve_symlinks (struct tarfs_entry *entry, 
-			  struct tarfs_loop_protect *list)
+static struct entry *
+__resolve_symlinks (struct entry *entry, 
+			  struct loop_protect *list)
 {
-    struct tarfs_entry *pent;
-    struct tarfs_loop_protect *looping;
+    struct entry *pent;
+    struct loop_protect *looping;
     
     if (!S_ISLNK (entry->inode->mode))
     	return entry;
@@ -911,155 +854,62 @@ __tarfs_resolve_symlinks (struct tarfs_entry *entry,
     	    errloop = 1;
     	    return NULL;
     	}
-    looping = (struct tarfs_loop_protect *)
-	xmalloc (sizeof (struct tarfs_loop_protect), 
+    looping = (struct loop_protect *)
+	xmalloc (sizeof (struct loop_protect), 
 		 "Tar: symlink looping protection");
     looping->entry = entry;
     looping->next = list;
-    pent = __tarfs_find_entry (entry->dir, entry->inode->linkname, looping, 0);
+    pent = __find_entry (entry->dir, entry->inode->linkname, looping, 0, 0);
     free (looping);
     if (pent == NULL)
-    	tarerrno = ENOENT;
+    	my_errno = ENOENT;
     return pent;
 }
 
-static struct tarfs_entry *tarfs_resolve_symlinks (struct tarfs_entry *entry)
+static struct entry *my_resolve_symlinks (struct entry *entry)
 {
-    struct tarfs_entry *res;
+    struct entry *res;
     
     errloop = 0;
     notadir = 0;
-    res = __tarfs_resolve_symlinks (entry, NULL);
+    res = __resolve_symlinks (entry, NULL);
     if (res == NULL) {
     	if (errloop)
-    	    tarerrno = ELOOP;
+    	    my_errno = ELOOP;
     	else if (notadir)
-    	    tarerrno = ENOTDIR;
+    	    my_errno = ENOTDIR;
     }
     return res;
 }
 
-static struct tarfs_entry*
-__tarfs_find_entry (struct tarfs_entry *dir, char *name, 
-		     struct tarfs_loop_protect *list, int make_dirs)
-{
-    struct tarfs_entry *pent, *pdir;
-    char *p, *q, *name_end;
-    char c;
-
-    if (*name == '/') { /* Handle absolute paths */
-    	name++;
-    	dir = dir->inode->archive->root_entry;
-    }
-
-    pent = dir;
-    p = name;
-    name_end = name + strlen (name);
-    q = strchr (p, '/');
-    c = '/';
-    if (!q)
-	q = strchr (p, 0);
-    
-    for (; pent != NULL && c && *p; ){
-	c = *q;
-	*q = 0;
-
-	if (strcmp (p, ".")){
-	    if (!strcmp (p, "..")) 
-		pent = pent->dir;
-	    else {
-		if ((pent = __tarfs_resolve_symlinks (pent, list))==NULL){
-		    *q = c;
-		    return NULL;
-		}
-		if (c == '/' && !S_ISDIR (pent->inode->mode)){
-		    *q = c;
-		    notadir = 1;
-		    return NULL;
-		}
-		pdir = pent;
-		for (pent = pent->inode->first_in_subdir; pent; pent = pent->next_in_dir)
-		    /* Hack: I keep the original semanthic unless
-		       q+1 would break in the strchr */
-		    if (!strcmp (pent->name, p)){
-			if (q + 1 > name_end){
-			    *q = c;
-			    notadir = !S_ISDIR (pent->inode->mode);
-			    return pent;
-			}
-			break;
-		    }
-		
-		/* When we load archive, we create automagically
-		 * non-existant directories
-		 */
-		if (pent == NULL && make_dirs) { 
-		    pent = generate_entry (dir->inode->archive, p, pdir, S_IFDIR | 0777);
-		}
-	    }
-	}
-	/* Next iteration */
-	*q = c;
-	p = q + 1;
-	q = strchr (p, '/');
-	if (!q)
-	    q = strchr (p, 0);
-    }
-    if (pent == NULL)
-    	tarerrno = ENOENT;
-    return pent;
-}
-
-static struct tarfs_entry *tarfs_find_entry (struct tarfs_entry *dir, char *name, int make_dirs)
-{
-    struct tarfs_entry *res;
-    
-    errloop = 0;
-    notadir = 0;
-    res = __tarfs_find_entry (dir, name, NULL, make_dirs);
-    if (res == NULL) {
-    	if (errloop)
-    	    tarerrno = ELOOP;
-    	else if (notadir)
-    	    tarerrno = ENOTDIR;
-    }
-    return res;
-}
-
-struct tar_pseudofile {
-    struct tarfs_archive *archive;
+struct pseudofile {
+    struct archive *archive;
     long pos;
     long begin;
     long end;
-    struct tarfs_entry *entry;
+    struct entry *entry;
 };
 
+static struct entry *find_entry (struct entry *dir, char *name, int make_dirs, int make_file);
+
 static void *tar_open (char *file, int flags, int mode)
 {
-    struct tar_pseudofile *tar_info;
-    struct tarfs_archive *archive;
-    char *p, *q;
-    struct tarfs_entry *entry;
+    struct pseudofile *tar_info;
+    struct archive *archive;
+    char *q;
+    struct entry *entry;
 
-    if ((p = tarfs_get_path (file, &archive, 0, 0)) == NULL)
+    if ((q = get_path_mangle (file, &archive, 0, 0)) == NULL)
 	return NULL;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    free (p);
+    entry = find_entry (archive->root_entry, q, 0, 0);
     if (entry == NULL)
     	return NULL;
-    if ((entry = tarfs_resolve_symlinks (entry)) == NULL)
+    if ((entry = my_resolve_symlinks (entry)) == NULL)
 	return NULL;
-    if (S_ISDIR (entry->inode->mode)) {
-    	tarerrno = EISDIR;
-    	return NULL;
-    }
-    if ((flags & O_ACCMODE) != O_RDONLY) {
-    	tarerrno = EROFS; /* At the moment we are RO */
-    	return NULL;
-    }
+    if (S_ISDIR (entry->inode->mode)) ERRNOR (EISDIR, NULL);
+    if ((flags & O_ACCMODE) != O_RDONLY) ERRNOR (EROFS, NULL);
     
-    tar_info = (struct tar_pseudofile *) xmalloc (sizeof (struct tar_pseudofile), "Tar: tar_open");
+    tar_info = (struct pseudofile *) xmalloc (sizeof (struct pseudofile), "Tar: tar_open");
     tar_info->archive = archive;
     tar_info->pos = 0;
     tar_info->begin = entry->inode->data_offset;
@@ -1075,14 +925,12 @@ static void *tar_open (char *file, int flags, int mode)
 
 static int tar_read (void *data, char *buffer, int count)
 {
-    struct tar_pseudofile *file = (struct tar_pseudofile *)data;
+    struct pseudofile *file = (struct pseudofile *)data;
 
     if (file->archive->is_gzipped != targz_growing && 
         mc_lseek (file->archive->fd, file->begin + file->pos, SEEK_SET) != 
-        file->begin + file->pos) {
-    	tarerrno = EIO;
-    	return -1;
-    }
+        file->begin + file->pos) ERRNOR (EIO, -1);
+
     
     if (count > file->end - file->begin - file->pos)
     	count = file->end - file->begin - file->pos;
@@ -1113,19 +961,16 @@ static int tar_read (void *data, char *buffer, int count)
             cnt -= j;
         }
     }
-    else if ((count = mc_read (file->archive->fd, buffer, count)) == -1) {
-    	tarerrno = errno;
-    	return -1;
-    }
+    else if ((count = mc_read (file->archive->fd, buffer, count)) == -1) ERRNOR (errno, -1);
     file->pos += count;
     return count;
 }
 
 static int tar_close (void *data)
 {
-    struct tar_pseudofile *file;
+    struct pseudofile *file;
 
-    file = (struct tar_pseudofile *)data;
+    file = (struct pseudofile *)data;
     
     file->archive->fd_usage--;
     if (!file->archive->fd_usage) {
@@ -1150,231 +995,41 @@ static int tar_close (void *data)
     return 0;
 }
 
-static int tar_errno (void)
-{
-    return tarerrno;
-}
-
-static void *tar_opendir (char *dirname)
-{
-    struct tarfs_archive *archive;
-    char *p, *q;
-    struct tarfs_entry *entry;
-    struct tarfs_entry **tar_info;
-
-    if ((p = tarfs_get_path (dirname, &archive, 1, 0)) == NULL)
-	return NULL;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    free (p);
-    if (entry == NULL)
-    	return NULL;
-    if ((entry = tarfs_resolve_symlinks (entry)) == NULL)
-	return NULL;
-    if (!S_ISDIR (entry->inode->mode)) {
-    	tarerrno = ENOTDIR;
-    	return NULL;
-    }
-
-    tar_info = (struct tarfs_entry **) xmalloc (sizeof (struct tarfs_entry *), "Tar: tar_opendir");
-    *tar_info = entry->inode->first_in_subdir;
-
-    return tar_info;
-}
-
-static void *tar_readdir (void *data)
-{
-    static struct {
-	struct dirent dir;
-#ifdef NEED_EXTRA_DIRENT_BUFFER
-	char extra_buffer [MC_MAXPATHLEN];
-#endif
-    } dir;
-    
-    struct tarfs_entry **tar_info = (struct tarfs_entry **) data;
-    
-    if (*tar_info == NULL)
-    	return NULL;
-    
-    strcpy (&(dir.dir.d_name [0]), (*tar_info)->name);
-    
-#ifndef DIRENT_LENGTH_COMPUTED
-    dir.d_namlen = strlen (dir.dir.d_name);
-#endif
-    *tar_info = (*tar_info)->next_in_dir;
-    
-    return (void *)&dir;
-}
-
-static int tar_closedir (void *data)
-{
-    free (data);
-    return 0;
-}
-
-static int _tar_stat (char *path, struct stat *buf, int resolve)
-{
-    struct tarfs_archive *archive;
-    char *p, *q;
-    struct tarfs_entry *entry;
-    struct tarfs_inode *inode;
-
-    if ((p = tarfs_get_path (path, &archive, 0, 0)) == NULL)
-	return -1;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    free (p);
-    if (entry == NULL)
-    	return -1;
-    if (resolve && (entry = tarfs_resolve_symlinks (entry)) == NULL)
-	return -1;
-    inode = entry->inode;
-    buf->st_dev = inode->dev;
-    buf->st_ino = inode->inode;
-    buf->st_mode = inode->mode;
-    buf->st_nlink = inode->nlink;
-    buf->st_uid = inode->uid;
-    buf->st_gid = inode->gid;
-#ifdef HAVE_ST_RDEV
-    buf->st_rdev = inode->rdev;
-#endif
-    buf->st_size = inode->size;
-#ifdef HAVE_ST_BLKSIZE
-    buf->st_blksize = RECORDSIZE;
-#endif
-#ifdef HAVE_ST_BLOCKS
-    buf->st_blocks = (inode->size + RECORDSIZE - 1) / RECORDSIZE;
-#endif
-    buf->st_atime = inode->atime;
-    buf->st_mtime = inode->mtime;
-    buf->st_ctime = inode->ctime;
-    return 0;
-}
-
-static int tar_stat (char *path, struct stat *buf)
-{
-    return _tar_stat (path, buf, 1);
-}
-
-static int tar_lstat (char *path, struct stat *buf)
-{
-    return _tar_stat (path, buf, 0);
-}
-
-static int tar_fstat (void *data, struct stat *buf)
-{
-    struct tar_pseudofile *file = (struct tar_pseudofile *)data;
-    struct tarfs_inode *inode;
-    
-    inode = file->entry->inode;
-    buf->st_dev = inode->dev;
-    buf->st_ino = inode->inode;
-    buf->st_mode = inode->mode;
-    buf->st_nlink = inode->nlink;
-    buf->st_uid = inode->uid;
-    buf->st_gid = inode->gid;
-    buf->st_rdev = inode->rdev;
-    buf->st_size = inode->size;
-#ifdef HAVE_ST_BLKSIZE
-    buf->st_blksize = RECORDSIZE;
-#endif
-#ifdef HAVE_ST_BLOCKS
-    buf->st_blocks = (inode->size + RECORDSIZE - 1) / RECORDSIZE;
-#endif
-    buf->st_atime = inode->atime;
-    buf->st_mtime = inode->mtime;
-    buf->st_ctime = inode->ctime;
-    return 0;
-}
+#include "shared_tar_ext.c"
 
 static int tar_chmod (char *path, int mode)
-{
+{	/* Fixme: are you sure? IMO this is guaranteed to fail */
     return chmod (path, mode);
 }
 
 static int tar_chown (char *path, int owner, int group)
-{
+{	/* Fixme: are you sure? IMO this is guaranteed to fail */
     return chown (path, owner, group);
 }
 
-static int tar_readlink (char *path, char *buf, int size)
-{
-    struct tarfs_archive *archive;
-    char *p, *q;
-    int i;
-    struct tarfs_entry *entry;
-
-    if ((p = tarfs_get_path (path, &archive, 0, 0)) == NULL)
-	return -1;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    free (p);
-    if (entry == NULL)
-    	return -1;
-    if (!S_ISLNK (entry->inode->mode)) {
-        tarerrno = EINVAL;
-        return -1;
-    }
-    if (size > (i = strlen (entry->inode->linkname))) {
-    	size = i;
-    }
-    strncpy (buf, entry->inode->linkname, i);
-    return i;
-}
-
-static int tar_unlink (char *path)
-{
-    return -1;
-}
-
-static int tar_symlink (char *n1, char *n2)
-{
-    return -1;
-}
-
-static int tar_write (void *data, char *buf, int nbyte)
-{
-    return -1;
-}
-
-static int tar_rename (char *a, char *b)
-{
-    return -1;
-}
-
 static int tar_chdir (char *path)
 {
-    struct tarfs_archive *archive;
-    char *p, *q;
-    struct tarfs_entry *entry;
+    struct archive *archive;
+    char *q;
+    struct entry *entry;
 
-    tarerrno = ENOTDIR;
-    if ((p = tarfs_get_path (path, &archive, 1, 0)) == NULL)
+    my_errno = ENOTDIR;
+    if ((q = get_path_mangle (path, &archive, 1, 0)) == NULL)
 	return -1;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    if (entry == NULL) {
-    	free (p);
+    entry = find_entry (archive->root_entry, q, 0, 0);
+    if (!entry)
     	return -1;
-    }
-    entry = tarfs_resolve_symlinks (entry);
-    if (entry == NULL) {
-    	free (p);
+    entry = my_resolve_symlinks (entry);
+    if ((!entry) || (!S_ISDIR (entry->inode->mode)))
     	return -1;
-    }
-    if (!S_ISDIR (entry->inode->mode)) {
-    	free (p);
-    	return -1;
-    }
     entry->inode->archive->current_dir = entry;
-    free (p);
-    tarerrno = 0;
+    my_errno = 0;
     return 0;
 }
 
 static int tar_lseek (void *data, off_t offset, int whence)
 {
-    struct tar_pseudofile *file = (struct tar_pseudofile *) data;
+    struct pseudofile *file = (struct pseudofile *) data;
 
     switch (whence) {
     	case SEEK_CUR:
@@ -1391,39 +1046,18 @@ static int tar_lseek (void *data, off_t offset, int whence)
     return file->pos;
 }
 
-static int tar_mknod (char *path, int mode, int dev)
-{
-    return -1;
-}
-
-static int tar_link (char *p1, char *p2)
-{
-    return -1;
-}
-
-static int tar_mkdir (char *path, mode_t mode)
-{
-    return -1;
-}
-
-static int tar_rmdir (char *path)
-{
-    return -1;
-}
-
 static vfsid tar_getid (char *path, struct vfs_stamping **parent)
 {
-    struct tarfs_archive *archive;
+    struct archive *archive;
     vfs *v;
-    vfsid id;
     char *p;
+    vfsid id;
     struct vfs_stamping *par;
 
     *parent = NULL;
-    if ((p = tarfs_get_path (path, &archive, 0, 1)) == NULL) {
+    if (!(p = get_path (path, &archive, 0, 1)))
 	return (vfsid) -1;
-    }
-    free (p);
+    free(p);
     v = vfs_type (archive->name);
     id = (*v->getid) (archive->name, &par);
     if (id != (vfsid)-1) {
@@ -1438,17 +1072,17 @@ static vfsid tar_getid (char *path, struct vfs_stamping **parent)
 
 static int tar_nothingisopen (vfsid id)
 {
-    if (((struct tarfs_archive *)id)->fd_usage <= 0)
+    if (((struct archive *)id)->fd_usage <= 0)
     	return 1;
     else
     	return 0;
 }
 
-static void free_entry (struct tarfs_entry *e)
+static void free_entry (struct entry *e)
 {
     int i = --(e->inode->nlink);
     if (S_ISDIR (e->inode->mode) && e->inode->first_in_subdir != NULL) {
-        struct tarfs_entry *f = e->inode->first_in_subdir;
+        struct entry *f = e->inode->first_in_subdir;
         
         e->inode->first_in_subdir = NULL;
         free_entry (f);
@@ -1470,8 +1104,8 @@ static void free_entry (struct tarfs_entry *e)
 
 static void tar_free (vfsid id)
 {
-    struct tarfs_archive *parc;
-    struct tarfs_archive *archive = (struct tarfs_archive *)id;
+    struct archive *parc;
+    struct archive *archive = (struct archive *)id;
 
     free_entry (archive->root_entry);
     if (archive == first_archive) {
@@ -1488,23 +1122,23 @@ static void tar_free (vfsid id)
 
 static char *tar_getlocalcopy (char *path)
 {
-    struct tarfs_archive *archive;
+    struct archive *archive;
     char *p, *q;
-    struct tarfs_entry *entry;
+    struct entry *entry;
+    char buf[MC_MAXPATHLEN];
 
-    if ((p = tarfs_get_path (path, &archive, 1, 0)) == NULL)
+    strcpy( buf, path );
+    if ((q = get_path_mangle (path, &archive, 1, 0)) == NULL)
 	return NULL;
-    q = (*p == '/') ? p + 1 : p;
-    entry = tarfs_find_entry (archive->root_entry, q, 0);
-    free (p);
+    entry = find_entry (archive->root_entry, q, 0, 0);
     if (entry == NULL)
     	return NULL;
-    if ((entry = tarfs_resolve_symlinks (entry)) == NULL)
+    if ((entry = my_resolve_symlinks (entry)) == NULL)
 	return NULL;
 
     if (entry->inode->local_filename != NULL)
         return entry->inode->local_filename;
-    p = mc_def_getlocalcopy (path);
+    p = mc_def_getlocalcopy (buf);
     if (p != NULL) {
         entry->inode->local_filename = p;
     }
@@ -1517,47 +1151,37 @@ static void tar_ungetlocalcopy (char *path, char *local, int has_changed)
    since it will be freed when tar archive will be freed */
 }
 
-#ifdef HAVE_MMAP
-caddr_t tar_mmap (caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
-{
-    return (caddr_t)-1;
-}
-
-int tar_munmap (caddr_t addr, size_t len, void *data)
-{
-    return -1;
-}
-#endif
-
 vfs tarfs_vfs_ops =
 {
     tar_open,
     tar_close,
     tar_read,
-    tar_write,
+    NULL,
 
-    tar_opendir,
-    tar_readdir,
-    tar_closedir,
+    s_opendir,
+    s_readdir,
+    s_closedir,
+    s_telldir,
+    s_seekdir,
 
-    tar_stat,
-    tar_lstat,
-    tar_fstat,
+    s_stat,
+    s_lstat,
+    s_fstat,
 
     tar_chmod,
     tar_chown,
     NULL,
 
-    tar_readlink,
-    tar_symlink,
-    tar_link,
-    tar_unlink,
+    s_readlink,
+    NULL,
+    NULL,
+    NULL,
 
-    tar_rename,
+    NULL,
     tar_chdir,
-    tar_errno,
+    s_errno,
     tar_lseek,
-    tar_mknod,
+    NULL,
     
     tar_getid,
     tar_nothingisopen,
@@ -1566,13 +1190,13 @@ vfs tarfs_vfs_ops =
     tar_getlocalcopy,
     tar_ungetlocalcopy,
     
-    tar_mkdir,
-    tar_rmdir,
+    NULL,		/* mkdir */
+    NULL,
     NULL,
     NULL,
     NULL
 #ifdef HAVE_MMAP
-    , tar_mmap,
-    tar_munmap
+    , NULL,
+    NULL
 #endif    
 };
