@@ -1,6 +1,17 @@
 #ifndef __VFS_H
 #define __VFS_H
 
+#ifdef VFS_STANDALONE
+#undef USE_EXT2FSLIB
+#else
+#define BROKEN_PATHS
+/*
+ * We should really only allow /#ftp/ tree to export ftp, but midnight's users may 
+ * like to be able to cd .. to get back where there were before ftp. How to solve?
+ * Ok, we'll allow /any/path/#ftp/ to access ftp tree. Broken, yes.
+ */ 
+#endif
+
 #if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H)
 #	include <sys/time.h>	/* alex: this redefines struct timeval */
 #endif /* SCO_FLAVOR */
@@ -15,7 +26,6 @@ struct utimbuf {
 #endif
 
 #ifdef USE_VFS
-
 #ifdef HAVE_MMAP
 #include <sys/mman.h>
 #endif
@@ -35,6 +45,8 @@ struct utimbuf {
 	void  *(*opendir)(char *dirname);
 	void  *(*readdir)(void *vfs_info);
 	int   (*closedir)(void *vfs_info);
+ 	int   (*telldir)(void *dir);
+ 	void  (*seekdir)(void *dir, int offset);
 
 	int  (*stat)(char *path, struct stat *buf);
 	int  (*lstat)(char *path, struct stat *buf);
@@ -78,9 +90,11 @@ struct utimbuf {
     extern vfs tarfs_vfs_ops;
 
     extern vfs ftpfs_vfs_ops;
+    extern vfs fish_vfs_ops;
     extern vfs mcfs_vfs_ops;
     
     extern vfs extfs_vfs_ops;
+    extern vfs sfs_vfs_ops;
 
     extern vfs undelfs_vfs_ops;
 
@@ -97,6 +111,7 @@ struct utimbuf {
 
     extern int vfs_type_absolute;
     vfs *vfs_type (char *path);
+    vfs *vfs_split (char *path, char **inpath, char **op);
     vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par);
     void vfs_rm_parents (struct vfs_stamping *stamp);
     char *vfs_path (char *path);
@@ -125,7 +140,14 @@ struct utimbuf {
     char *tarfs_analysis (char *inname, char **archive, int is_dir);
 
     void ftpfs_init(void);
+    void fish_init(void);
+    void sfs_init(void);
     void ftpfs_done(void);
+    void sfs_done(void);
+    void fish_done(void);
+    int extfs_which (char *path);
+    int sfs_which (char *path);
+
     void ftpfs_set_debug (char *file);
 #ifdef USE_NETCODE
     void ftpfs_hint_reread(int reread);
@@ -138,13 +160,9 @@ struct utimbuf {
     void mcfs_fill_names (void (*)(char *));
     void ftpfs_fill_names (void (*)(char *));
     void tarfs_fill_names (void (*)(char *));
+    void fish_fill_names (void (*)(char *));
+    void sfs_fill_names (void (*)(char *));
     
-    char *ftpfs_gethome (char *);
-    char *mcfs_gethome (char *);
-    
-    char *ftpfs_getupdir (char *);
-    char *mcfs_getupdir (char *);
-
     /* Only the routines outside of the VFS module need the emulation macros */
 
 	int mc_open (char *file, int flags, ...);
@@ -157,6 +175,8 @@ struct utimbuf {
 	DIR *mc_opendir (char *dirname);
 	struct dirent *mc_readdir(DIR *dirp);
 	int mc_closedir (DIR *dir);
+ 	int mc_telldir (DIR *dir);
+ 	void mc_seekdir (DIR *dir, int offset);
 
 	int mc_stat (char *path, struct stat *buf);
 	int mc_lstat (char *path, struct stat *buf);
@@ -192,6 +212,8 @@ struct utimbuf {
 #    undef USE_NETCODE
 #endif
 
+#    undef USE_NETCODE
+
 #   define vfs_fill_names(x)
 #   define vfs_add_current_stamps()
 #   define vfs_current_is_local() 1
@@ -207,6 +229,8 @@ struct utimbuf {
 #   define mc_opendir opendir
 #   define mc_readdir readdir
 #   define mc_closedir closedir
+#   define mc_telldir telldir
+#   define mc_seekdir seekdir
 
 #   define mc_get_current_wd(x,size) get_current_wd (x, size)
 #   define mc_fstat fstat
@@ -275,26 +299,83 @@ struct utimbuf {
 
 #ifdef WANT_PARSE_LS_LGA
 int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname);
+int split_text (char *p);
+int parse_filetype (char c);
+int parse_filemode (char *p);
+int parse_filedate(int idx, time_t *t);
 #endif
+extern void vfs_die (char *msg);
+extern char *vfs_get_password (char *msg);
 
 #define MCCTL_SETREMOTECOPY	0
 #define MCCTL_ISREMOTECOPY	1
 #define MCCTL_REMOTECOPYCHUNK	2
 #define MCCTL_FINISHREMOTE	3
-#define MCCTL_FLUSHDIR          4
+#define MCCTL_FLUSHDIR          4	/* Unreferenced */
+#define MCCTL_REMOVELOCALCOPY   5
+#define MCCTL_IS_NOTREADY	6
 
 /* Return codes from the ${fs}_ctl routine */
 
-#define MCERR_TARGETOPEN	-1
-    /* Can't open target file */
-#define MCERR_READ		-2
-    /* Read error on source file */
-#define MCERR_WRITE		-3
-    /* Write error on target file */
-#define MCERR_FINISH		-4
-    /* Finished transfer */
-#define MCERR_DATA_ON_STDIN     -5
-    /* Data waiting on stdin to be processed */
+#define MCERR_TARGETOPEN	-1    /* Can't open target file */
+#define MCERR_READ		-2    /* Read error on source file */
+#define MCERR_WRITE		-3    /* Write error on target file */
+#define MCERR_FINISH		-4    /* Finished transfer */
+#define MCERR_DATA_ON_STDIN     -5    /* Data waiting on stdin to be processed */
+
+extern int vfs_flags;
+extern uid_t vfs_uid;
+extern gid_t vfs_gid;
+
+#define FL_ALWAYS_MAGIC 1
+#define FL_NO_MCFS 2
+#define FL_NO_FTPFS 4
+#define FL_NO_UNDELFS 8
+#define FL_NO_TARFS 16
+#define FL_NO_EXTFS 32
+#define FL_NO_SFS 64
+#define FL_NO_FISH 128
+
+#define FL_NO_CWDSETUP	0x40000000
+
+#ifdef VFS_STANDALONE
+extern void mc_vfs_init( void );
+extern void mc_vfs_done( void );
+#endif
+
+#define ERRNOR(x,y) do { my_errno = x; return y; } while(0)
+
+#define O_ALL (O_CREAT | O_EXCL | O_NOCTTY | O_NDELAY | O_SYNC | O_WRONLY | O_RDWR | O_RDONLY)
+/* Midnight commander code should _not_ use other flags than those
+   listed above and O_APPEND */
+
+#if (O_ALL & O_APPEND)
+#warning Unexpected problem with flags, O_LINEAR disabled, contact pavel@ucw.cz
+#define O_LINEAR 0
+#define IS_LINEAR(a) 0
+#define NO_LINEAR(a) a
+#else
+#define O_LINEAR O_APPEND
+#define IS_LINEAR(a) ((a) == (O_RDONLY | O_LINEAR))
+#define NO_LINEAR(a) (((a) == (O_RDONLY | O_LINEAR)) ? O_RDONLY : (a))
+#endif
+
+/* O_LINEAR is strange beast, be carefull. If you open file asserting
+ * O_RDONLY | O_LINEAR, you promise:
+ *
+ *     	a) to read file linearily from beggining to the end
+ *	b) not to open another file before you close this one
+ *		(this will likely go away in future)
+ *
+ * O_LINEAR allows filesystems not to create temporary file in some
+ * cases (ftp transfer).				-- pavel@ucw.cz
+ */
+
+#ifndef MIN
+#define MIN(a,b) ((a)<(b) ? (a) : (b))
+#endif
+#ifndef MAX
+#define MAX(a,b) ((a)<(b) ? (b) : (a))
+#endif
 
 #endif /* __VFS_H */
-
